//****************************************************************
//
//    Class dense
//
//    Description:
//      Dense is a class for implementing dense matrices.
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
//****************************************************************

/*==========================================================================
				PRIVATE :
				--------

    int nrow;		// row dimension of the matrix
    int ncol;		// col dimension of the matrix
    double *matrix;	// double array which contains the values of the matrix

				FUNCTIONS :
				---------
constructors
destructor

get_pointer : 		returns a pointer to the matrix
get_nrow : 		returns the row dimension of a matrix
get_ncol : 		returns the col dimension of a matrix
get_nnz : 		returns the number of non zero element of a matrix
get_element(i,j) : 	returns the element of the matrix with the coordinates
			(i,j)
set_element(i,j,v) : 	sets the element (i,j) to the value v
initialize(x) : 	sets all the elements of the matrix to the value x
make_identity : 	makes the matrix into an identity matrix
submat :		extracts a submatrix from a dense one.
getdiag : 		extracts a specified diagonal from a matrix.
rmdiag : 		removes a specified diagonal from a matrix.
getu :			extracts the upper triangular part of a matrix
getl :			extracts the lower triangular part of a matrix
getbwd :	        gives the bandwith of the upper and the lower part 
			of the mat
getbwdu :	        gives the bandwith of the upper part of the matrix
getbwdl :	        gives the bandwith of the lower part of the matrix


operators: 		=, <<= , !, +, -, *, &&, ||, <, <=, >, >=, ==, !=

filter(mat,f) : 	Returns a new matrix where each element (i,j) is
			calculated by applying the function to the i,j element
			of mat.
transp : 		calculates the transpose of a matrix
emult : 		multiplies 2 matrices element by element
ediv : 			divides 2 matrices element by element
dot : 			if it is possible, calculates the dot product
output : 		outputs the content of a matrix


==========================================================================*/





#ifndef DENSE_H
#define DENSE_H

#include <iostream.h> 
#include <assert.h>
#include <stdlib.h>
#include "sparse.h"

extern "C" { typedef double (*ffunc)(double); }


class dense {

  public:
    // Constructors
    dense ();
    dense (int, int);
    dense (int, int, double);
    dense (sparse &);
    dense (const dense &);

    void free();

    // Destructor
    ~dense();

    // Selectors
    double get_element (int, int) const;
    int size();
    double& get_element_ref (int, int);
    double* get_pointer ();
    int get_nrow () const;
    int get_ncol () const;
    int get_nnz () const;

    // Member functions
    void set_element (int, int, double);
    void initialize_zero ();
    void make_identity ();
    void initialize (double);

    // Member operators
    void operator = (const dense &);
    void operator <<= (dense &);
    void submat(int, int, int, int, int);
    void getdiag(int);
    void rmdiag(int);
    void getu();
    void getl();
    int * getbwd();
    int getbwdu();
    int getbwdl();
    double * cnrms(int);
    double * rnrms(int);
    void coscal(int);
    void roscal(int);
    void rando(int);
    double * retmax();

  private:
    // Data
    int nrow;		// row dimension of the matrix
    int ncol;		// col dimension of the matrix
    double *matrix;	// double array which contains the values of the matrix

}; // end class dense


void output(dense &);
int absol(int);
double dot (const dense &, const dense &);
dense filter (const dense &, ffunc);
dense transp (const dense &);
dense emult (const dense &, const dense &);
dense ediv (const dense &, const dense &);
dense operator ! (const dense &);
dense operator + (const dense &, const dense &);
dense operator + (const double &, const dense &);
dense operator + (const dense &, const double &);
dense operator - (const dense &, const dense &);
dense operator - (const double &, const dense &);
dense operator - (const dense &, const double &);
dense operator - (const dense &);
dense operator * (const dense &, const dense &);
dense operator * (const dense &, const double &);
dense operator * (const double &, const dense &);
dense operator / (const dense &, const double &);
dense operator && (const dense &, const double &);
dense operator && (const double &, const dense &);
dense operator && (const dense &, const dense &);
dense operator || (const dense &, const double &);
dense operator || (const double &, const dense &);
dense operator || (const dense &, const dense &);
dense operator < (const dense &, const double &);
dense operator < (const double &, const dense &);
dense operator < (const dense &, const dense &);
dense operator <= (const dense &, const double &);
dense operator <= (const double &, const dense &);
dense operator <= (const dense &, const dense &);
dense operator == (const dense &, const double &);
dense operator == (const double &, const dense &);
dense operator == (const dense &, const dense &);
dense operator != (const dense &, const double &);
dense operator != (const double &, const dense &);
dense operator != (const dense &, const dense &);
dense operator >= (const dense &, const double &);
dense operator >= (const double &, const dense &);
dense operator >= (const dense &, const dense &);
dense operator > (const dense &, const double &);
dense operator > (const double &, const dense &);
dense operator > (const dense &, const dense &);

dense cperm (dense &, int *);
dense rperm (dense &, int *);
dense dperm (dense &, int *, int *);


#endif
