// ****************************************************************
//
//    Class entry
//
//    Description:
//      Entry is a class for implementing an entry in a symbol
//      table.  Each entry is composed of a string pointer
//      and a value.  (See class value.)
//	
//     Remark : spmath will use entries to store the variables.	
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
// ****************************************************************

/*==========================================================================
				PRIVATE :
				--------

    char *name;		// name of the entry
    value data;		// value of the entry


				FUNCTIONS :
				----------

constructors
destructor

get_name : 		returns the pointer to the name 
get_value : 		returns the value component of the entry
set_name(n) : 		sets the name of the entry to the string pointed to by n
set_value_str(n) : 	sets the value of the entry to the string pointed to by
			n
set_value_num(n) : 	sets the value of the entry to the number n
set_value_dense(n) :	sets the value of the entry to the dense matrix n
set_value_sparse(n) : 	sets the value of the entry to the sparse matrix n
operator =

==========================================================================*/

#include "entry.h"

//******************************************************************
//       Member Constructors
//******************************************************************


entry::entry() {
  // Empty constructor to create an empty symbol table entry

  data.type = NO_TYPE;
} // end constructor entry

entry::entry (char *n) {
  // Constructor to put a name into the symbol table entry
  // without a value.

  name = n;
  data.type = NO_TYPE;
} // end constructor entry

entry::entry (char *n, value v) {
  // Constructor to put a name into the symbol table entry
  // with the value v.

  name = n;
  data = v;
} // end constructor entry

entry::entry (const entry &x) {
  // Copy constructor

  name = x.get_name();
  data = x.get_value();
} // end constructor entry

//******************************************************************
//       Member Selectors
//******************************************************************

/*=========================================================================
			GET_NAME :
			---------
Entries : none
Returns : the pointer to the name of the entry

=========================================================================*/

char * entry::get_name() const {
  // Return the pointer to the name string.

  return name;

} // end selector get_name

/*=========================================================================
			GET_VALUE :
			----------
Entries : none
Returns : the value of the entry which can be either a double, a string, 
	  a dense matrix, a sparse matrix or a function.

=========================================================================*/

value entry::get_value() const {
  // Return the value component of the symbol table entry.

  return data;

} // end selector get_value


//******************************************************************
//       Member Mutators
//******************************************************************

/*=========================================================================
			SET_NAME :
			----------
Entries : a name (a character) :n
Returns : nothing
Details : sets the name of the entry to the string pointed to by n

=========================================================================*/

void entry::set_name(char *n) {
  // Set the name of the entry to n the string pointed to by n.

  name = n;

} // end selector set_name

/*=========================================================================
			SET_VALUE_STR :
			-------------
Entries : a pointer to a character : n
Returns : nothing
Details : sets the value of the entry to the string pointed to by n

=========================================================================*/

void entry::set_value_str (char *n) {
  // Set the value to the string pointed to by n

  data.str = n;
  data.type = STRING;

} // end mutator set_value_str

/*=========================================================================
			SET_VALUE_NUM :
			-------------
Entries : a double : n
Returns : nothing
Details : sets the value of the entry to double n

=========================================================================*/

void entry::set_value_num (double n) {
  // Set the value to the number n.

  data.num = n;
  data.type = NUM;

} // end mutator set_value_num

/*=========================================================================
			SET_VALUE_DENSE :
			---------------
Entries : a dense matrix d
Returns : nothing
Details : sets the value of the entry to the dense matrix d

=========================================================================*/

void entry::set_value_dense (dense *d) {
  // Set the value to the dense matrix d.

  data.dmat = d;
  data.type = DENSE;

} // end mutator set_value_dense

/*=========================================================================
			SET_VALUE_SPARSE :
			----------------
Entries : a sparse matrix d
Returns : nothing
Details : sets the value of the entry to the sparse matrix d

=========================================================================*/

void entry::set_value_sparse (sparse *s) {
  // Set the value to the sparse matrix  s.

  data.smat = s;
  data.type = SPARSE;

} // end mutator set_value_sparse

void entry::free () {
  // Free the dynamically allocated memory


  // Free up the memory for the name.
  delete [] name;
  // Free up the memory from the value.
  data.free();

} // end free

//******************************************************************
//       Member Operators
//******************************************************************

/*============================================================================
#				OPERATOR =		
#				----------    	
# in entry  : x -> an entry : x
#
# in return : an entry : result
#
# details   : This is an assignment operator result = x
#
============================================================================*/

entry entry::operator = (const entry & x) {
  // Assignment operator

  name = x.get_name();
  data = x.get_value();

  return x;

} // end operator =


