//////////////////////////////////////////////////////////
//
// main.C
// ~~~~~~
//  This source file defines the basic startup and
// shutdown routines for spmath.
//
// Author: Ben Langmead (btl10@columbia.edu)
// Date:   June 29, 2001
//
//////////////////////////////////////////////////////////

#include <string.h>
#include "spmath.h"

#ifndef NO_TK_COMPILE

// Useful tk globals
Tk_Window   Top;
Display	   *Disp;
Tcl_Interp *interp;

#endif

char spmathpath[128];

int no_tk = 0;

// Prototypes
void sp_quit(int);

#ifndef NO_TK_COMPILE

int  process_args(int, char**);

int  Tcl_AppInit();
int  tcl_init();
int  program_init();
int  window_init();

void TCL(const char *);

#endif

//
// int main(int argc, char *argv[])
//
//  The main program.  First this reads in the arguments
// and then calls process_arguments to set the global
// flags according to the user's input.  Then, depending
// on those flags, initializes the app accordingly.
//

int main(int argc, char *argv[]) {

  // Get the directory that spmath is in so that we can
  // find the tcl scripts.

  strncpy(spmathpath, argv[0], 127);
  char *cut = spmathpath;

  while(strstr(cut, "spmath") != NULL) {

    cut = strstr(cut, "spmath");
    cut++;
    if(cut[0] == '\0') break;
  } 
  cut --;
  cut[0] = '\0';
  

#ifndef NO_TK_COMPILE
  if (!process_args(argc, argv)) {

    // If the args were malformed, give user usage 
    // instructions

    cerr << "Usage: " << endl
	 << "  spmath [-notk]" << endl
	 << endl
	 << "  -notk  Uses command-line only, without tk." << endl
	 << endl;

    exit(1);
  }

  if(no_tk) {

    sp_init();

    // Let the parser take over (control will never
    // return to the main function)
    sp_parse(NULL);
  } 
  else {

    // Let tk take over (control never returns to the
    // main function)
    Tk_Main( argc, argv, Tcl_AppInit );
  }
#else

  sp_init();
  
  // Let the parser take over (control will never
  // return to the main function)
  sp_parse(NULL);
  
#endif

  // This code will never be reached
  return( 0 );
}

//
// sp_quit(int ret)
//
//  Quits from either tk or no_tk mode with return value
// ret.
//

void sp_quit(int ret) {

#ifndef NO_TK_COMPILE

  // Tell tk app to quit
  if(!no_tk) {
    TCL( "exit");
  }

#endif
  
  exit(ret);
}

#ifndef NO_TK_COMPILE

//
// TCL(char *e)
//
//  Sends the command "e" to the tcl interpretter and
// outputs an error if one occurs.
//

void TCL(const char *e) {

  if(!no_tk) {
    if(Tcl_Eval(interp, strdup(e)) == TCL_ERROR) {
      cout << "Tcl_Error evaluating \"" << e << "\"." << endl;
      // Output tcl's error message
      cout << interp->result << endl;
    }
  }
  else {
    cerr << "Internal error, received request to execute tcl command:" << endl
	 << e << endl
	 << "...cannot execute tcl commands in text mode" << endl;
  }
}

//
// Tcl_AppInit(Tcl_Interp *itp)
//
//  Saves itp in a global variable, interp, so that
// we can use the interpretter object throughout the
// code.  Also calls all the extra initialization
// functions needed for tk mode.
//

int Tcl_AppInit(Tcl_Interp *itp) {

  /* Save Interpreter */
  interp = itp;

  /* Initialize TCL & TK */
  if ( tcl_init() == TCL_ERROR ) {
    return( TCL_ERROR );
  }

  /* Get Main Window */
  Top = Tk_MainWindow( interp );
  if ( Top == NULL )
    return( TCL_ERROR );

  Disp = Tk_Display(Top);

  // Initializes the big view array
  if ( program_init() == TCL_ERROR ) {
    return( TCL_ERROR );
  }

  // Runs spmath.tcl to open the main window
  if ( window_init() == TCL_ERROR ) {
    Tcl_SetResult(interp, strdup("window_init() failed"), TCL_STATIC);
    return( TCL_ERROR );
  }

  // Initialize the windowed environment by setting
  // the golbal constants, displaying the welcome
  // message...
  sp_init();

  // Issue a prompt
  TCL("put_prompt");

  return( TCL_OK );
}

//
// int program_init()
//
//   Initializes the gobal array of views.  Called
// only if tk is on.
//

int program_init() {
  
  int i;
  for(i = 0; i < MAX_VIEWS; i++) {
    if(spyviews[i] != NULL) {
      spyviews[i] = NULL;
    }
  }
  cur_spyview = 0;

  return(TCL_OK);
}

//
// process_args(int argc, char **argv)
//
//  Checks the command line arguments for valid
// flags
//

int process_args(int argc, char **argv) {

  int i;

  for (i = 1 ; i < argc ; i++) {

    // Check for the -notk argument, which activates the
    // command-line only interface

    if ( !strcmp( argv[i], "-notk" )) {
      cout << ">> command-line mode..." << endl;

      no_tk = 1;
    }
    else {
      cerr << endl << "Bad flag: " << argv[i] << endl << endl;

      return 0;
    }
  }

  return 1;
}

//
// tcl_init()
//
//  Initializes the actual tcl and tk interpretters
// so that we can feed them our scripts.  Also calls
// create_tcl_commands in commands.C which registers
// all C++ functions to be called from tcl.
//

int tcl_init()
{
  /* Initialize TCL / TK */

  if ( Tcl_Init( interp ) == TCL_ERROR ) {
    cout << "Tcl_Init returned error: " << endl;
    cout << interp->result << endl;
    return(TCL_ERROR);
  }

  if ( Tk_Init( interp ) == TCL_ERROR ) {
    cout << "Tk_Init returned error: " << endl;
    cout << interp->result << endl;
    return(TCL_ERROR);
  }

  create_tcl_commands();
  
  return (TCL_OK);
}

//
// window_init()
//
//  Initializes the main tcl script which opens the
// command interpretter window.
//

int window_init()
{
  char fname[1024];

  strcpy( fname, spmathpath );
  strcat( fname, "spmath.tcl" );

  if ( Tcl_EvalFile( interp, fname ) == TCL_ERROR ) {
    cout << "Tcl_EvalFile returned an error:" << endl;
    cout << interp->result << endl;
    cout << "Quitting..." << endl;
    return(TCL_ERROR);
  }

  return(TCL_OK);
}

#endif





