%{ 

//////////////////////////////////////////////////////////
//
// spmath.y
// ~~~~~~~~
//
//  This is the yacc grammar file for spmath.  It contains
// the definition of spmath's grammar, as well as some
// important utility functions for loading, parsing, and
// saving spmath commands and files.  There are also a few
// utility math functions.
//
//  The first section ,everything from the beginning of the
// file to the first "%%", contains definitions and 
// prototypes.  It is also where all the lex tokens and
// precedence rules are set.  
//
//  The second section, everything from the first "%%" to
// the second "%%", contains the actual grammar.  
// Basically, this consists of a context free grammar in
// a variation of backus-naur form.  The rules have the
// form
//
// Variable: (Variables + tokens)
//         | (Variables + tokens)
//         | (Variables + tokens)
//         etc...
//         ;
//
// Note that each rule has a "if(text_flag)" section and
// then an "else" section.  This is because when text_flag
// is true, the parser is in the process of parsing a 
// user-defined function's body, which is being turned 
// into text so it can be parsed later when it is called 
// by the user.  When text_flag is false, the command is
// simply being executed.
//
// See a good lex/yacc book for more info on grammars,
// and also look at README_MODIFICATION.  Also, see 
// functions.C for the definitions of many of the 
// fucntions called from these grammar rules.
//
//  The third section, everything from the second "%%" to
// the end of the file, contains some C function 
// definitions.  Everything between the "%{" and "%}"
// symbols is copied verbatim into the output C file.
//
//////////////////////////////////////////////////////////

#include <stdlib.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <assert.h>
#include "spmath.h"

  // Size of the input buffer

#define BUFF_SIZE 8*1024

  // A helpful yacc define

#define YYERROR_VERBOSE

  // A few helpful lex defines

#define CATCH_MODE 1
#define VECTOR_MODE 2

int yyerror(char *);
int yyerror(const char *);
int yyparse();
int sp_parse(char *input);
int sp_parse_string(const char *);
value * load (value *, int, int &);
value * save (value *, value *, int &);
value * eval_funct (char *, list *, int &);
value * process_comma_list (list *, int &);
value * process_dense_row_list (list *, int &);

// These are defined to be extern "C" so they can be
// passed as "ffunc"s to the "function" function in
// functions.C

extern "C" {
  double cot (double);
  double csc (double);
  double sec (double);
  double acot (double);
  double acsc (double);
  double asec (double);
  double round (double);
  double trunc (double);
  double rnd (double);
}

symtab root, alia;
symtab *syms = &root;
int hist_num = 1; 
int hist_cur = 1;

// The global history list
history hist_list (100);

// global variables for using a string for input to the lexer.
char *str;
int lk;
int clear_alias = 0;   // Allow to remove an alias!
int several_init = 0;  // 1 when the arguments of a function are in fact output
int str_flag = 0;
symtab *ali = &alia;
int text_flag = 0;
int nest_count = 0;
 int supress_output = 0;
int err = 0;

extern FILE *yyin;
extern void yyrestart (FILE *);
extern void my_yybegin(int);

// The list of view for visualization in tk mode

#ifndef NO_TK_COMPILE

view *spyviews[MAX_VIEWS];
int cur_spyview;

#endif

%}

%pure_parser

%union {
  char *name;
  value *val;
  list *set;
}

%{

extern int yy_start;
extern int yylex(YYSTYPE *);

%}

    /*

    Precedence rules follow.  See a lex/yacc book for
    more information about precedence rules in grammars

    */

%left     SEMI COMMA
%right    ASSIGN
%left     OR AND
%nonassoc LT LE EQ GE GT NE
%left     COLON
%left     PLUS MINUS
%left     MULT EMULT DIV EDIV LDIV ELDIV
%nonassoc TRANSP
%nonassoc UPLUS UMINUS INC DEC NOT
%right    POW EPOW

   /*
 
   All the tokens follow.  To add a new keyword, you must
   add a new token here.  For example if you added a function
   named "add" you would add a line here that says 

   %token ADD

   and then change spmath.l to include a rule for detecting
   the string "add" like this:

   "add"  { return ADD; }

   */   

%token ERROR

%token TYPE
%token WHO
%token WHOS

%token BREAK
%token CONTINUE
%token ELSE
%token ELSEIF
%token END
%token ENDFOR
%token ENDFUNCTION
%token ENDIF
%token ENDWHILE
%token FOR
%token FUNCTION
%token GLOBAL
%token IF
%token RETURN
%token WHILE
%token HISTORY
%token HIST_LIST

%token LOAD
%token DEMO
%token READMT
%token GEN57PT
%token SUBMAT
%token EXTRACT
%token EXTRACT_COL
%token EXTRACT_ROW
%token FILTER
%token CSORT
%token RMDIA
%token GETDIA
%token GETU
%token GETL
%token GETBWD
%token GETBWDU
%token GETBWDL
%token BLKFND
%token CNRMS
%token ROSCAL
%token COSCAL
%token ADDBLK
%token GET1UP
%token PRTMT
%token RPERM
%token CPERM
%token DPERM
%token HELP
%token SOLVE_SYSTEM
%token MULTIC
%token COLORS
%token COLORS2
%token COLORSET
%token INDSET0
%token INDSET1

%token SPY

%token SQRT
%token COS
%token SIN
%token TAN
%token COT
%token SC
%token CSC
%token ARCCOS
%token ARCSIN
%token ARCTAN
%token ARCCOT
%token ARCSEC
%token ARCCSC
%token ABS
%token EXP
%token LOG
%token LOG10
%token RND
%token IRND
%token CEIL
%token FLOOR
%token ROUND
%token TRUNC
%token NCOL
%token NROW
%token NNZ
%token SIZE2
%token EYE
%token ZEROS
%token SPZEROS
%token ONES
%token INIT
%token INPUT
%token CLEAR
%token DENSIFY
%token SPARSIFY
%token ISDENSE
%token ISNUMBER
%token ISSPARSE
%token ISSTRING
%token ISVECTOR
%token WHAT_IS

%token ALIAS

    /*

    These declare the types of the terminals

    */

%token <name> NAME
%token <val> NUMBER
%token <val> STR
%token <name> BODY
%token <name> CATCH
%token <name> SYSTEM

    /*

    These declare the types of the non-terminals

    */

%type <val> control_statement
%type <val> while_statement
%type <val> for_statement
%type <val> if_statement
%type <val> function_definition
%type <val> expression
%type <val> statement
%type <set> comma_list
%type <val> dense_row
%type <val> dense_row_end
%type <set> dense_row_list
%type <val> dense
%type <val> vector
%type <val> function_call

%% 

statement: expression SEMI statement {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ";");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    $1->free ();
    delete $1;
  }				// end else
}

| expression COMMA statement {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ",");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    output ($1);
    sp_out("\n");
    $1->free ();
    delete $1;
  }				// end else
}

| control_statement SEMI statement {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ";");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    $1->free ();
    delete $1;
  }				// end else
}

| control_statement COMMA statement {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ";");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    output ($1);
    sp_out("\n");
    $1->free ();
    delete $1;
  }				// end else
}

| expression {
  if (text_flag)
    $$ = $1;
  else {
    output ($1);
    sp_out("\n");
    $1->free ();
    delete $1;
  }
}

| control_statement {
  if (text_flag)
    $$ = $1;
  else {
    output ($1);
    sp_out("\n");
    $1->free ();
    delete $1;
  }				// end else
}

 // \$1

| SYSTEM {
  if (text_flag)
    $$ = new value ($1);
  else
    system ($1 + 1);
}

 // history $2

| HISTORY expression {
  if (text_flag) {
    sp_error("Error, can't reference history list inside an expression.\n");
    YYERROR;
  }				// end if
  if ($2->type != NUM) {
    sp_error("Error, reference to history list must be a number.\n");
    YYERROR;
  }				// end if
  str_flag = 1;
  str =
    new char[strlen (hist_list.get_item (int ($2->num)).get_pointer ()) + 1];
  strcpy (str, hist_list.get_item (int ($2->num)).get_pointer ());
  strcat (str, "$");
  yyrestart (yyin);
  yyparse ();
}

 // "history"

| HIST_LIST {
  if (text_flag) {
    $$ = new value (strdup ("history"));
  }
  else {
    output (hist_list);
    $$ = new value ();
  }
}

 // "alias"

| ALIAS {
  if (text_flag) {
    $$ = new value (strdup ("alias"));
  }
  else {
    ali->out_alias ();
    $$ = new value ();
  }
}

 // "who"

| WHO {
  if (text_flag) {
    $$ = new value(strdup ("who"));
  }
  else {
    syms->out();
    $$ = new value();
  }
}

 // "whos"

| WHOS {
  if (text_flag) {
    $$ = new value(strdup ("whos"));
  }
  else {
    syms->out_fancy();
    $$ = new value();
  }
}

|  {
  if (text_flag)
    $$ = new value (strdup (""));
}

| error {
  yyrestart (yyin);
  text_flag = 0;
  str_flag = 0;
  return 0;
}

;


control_statement: for_statement {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

| if_statement {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

| while_statement {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

| function_definition {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

;

 // NAME=NAME(...) $9 end

function_definition: FUNCTION NAME ASSIGN NAME '(' { 
  text_flag += 1;
} comma_list ')' statement { 
  text_flag -= 1; 
} END {
  if (text_flag) {
    $7->init_curr_prev ();
    int len = strlen ($7->get_current ().get_value ().str) + 1;
    while (!$7->end ()) {
      $7->get_next ();
      len += strlen ($7->get_current ().get_value ().str) + 1;
    }				// end while
    char *temp =
      new char[8 + strlen ($2) + 1 + strlen ($4) + 1 + len + 1 +
	       strlen ($9->str) + 3];
    $7->init_curr_prev ();
    strcpy (temp, "function");
    strcat (temp, $2);
    strcpy (temp, "=");
    strcat (temp, $4);
    strcpy (temp, "(");
    strcat (temp, $7->get_current ().get_value ().str);
    while (!$7->end ()) {
      $7->get_next ();
      strcat (temp, ",");
      strcat (temp, $7->get_current ().get_value ().str);
    }				// end while
    strcpy (temp, ")");
    strcat (temp, $9->str);
    strcpy (temp, "end");
    $$ = new value (temp);
    $7->remove_all ();
    delete $7;
    delete[] $2;
    delete[] $4;
    $9->free ();
    delete $9;
  }				// end if
  else {
    $7->init_curr_prev ();
    int len = strlen ($7->get_current ().get_value ().str) + 1;
    while (!$7->end ()) {
      $7->get_next ();
      len += strlen ($7->get_current ().get_value ().str) + 1;
    }				// end while
    char *temp = new char[1 + len + 1];
    $7->init_curr_prev ();
    strcpy (temp, "(");
    strcat (temp, $7->get_current ().get_value ().str);
    while (!$7->end ()) {
      $7->get_next ();
      strcat (temp, ",");
      strcat (temp, $7->get_current ().get_value ().str);
    }				// end while
    strcat (temp, ")");
    root.insert (entry ($4, value (new funct_def ($2, temp, $9->str))));
    $$ = new value ();
    $7->remove_all ();
    delete $7;
    delete $9;
  }				// end else
}

| FUNCTION NAME ASSIGN NAME '(' { 
  text_flag += 1; 
} ')' statement { 
  text_flag -= 1; 
} END {
  if (text_flag) {
    char *temp =
      new char[8 + strlen ($2) + 1 + strlen ($4) + 1 + 1 + strlen ($8->str) +
	       3];
    strcpy (temp, "function");
    strcat (temp, $2);
    strcpy (temp, "=");
    strcat (temp, $4);
    strcpy (temp, "()");
    strcat (temp, $8->str);
    strcpy (temp, "end");
    $$ = new value (temp);
    delete[] $2;
    delete[] $4;
    $8->free ();
    delete $8;
  }				// end if
  else {
    char *temp = new char[1 + 1];
    strcpy (temp, "()");
    root.insert (entry ($4, value (new funct_def ($2, temp, $8->str))));
    $$ = new value ();
    delete $8;
  }				// end else
}


;

while_statement: WHILE '(' { 
  text_flag += 1; 
} expression { 
  text_flag -= 1; 
} ')' { 
  text_flag += 1; 
} statement { 
  text_flag -= 1; 
} END { 
  // Send lex into "catch-all" mode where it reads everything
  // up the next dollar sign and then returns it as a string
  // (without tokenizing it).
  my_yybegin(CATCH_MODE);   
} CATCH {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($4->str) + 1 +
			  strlen ($8->str) + 3 + strlen ($12)];
    strcpy (temp, "while(");
    strcat (temp, $4->str);
    strcat (temp, ")");
    strcat (temp, $8->str);
    strcat (temp, "end");
    strcat (temp, $12);
    $$ = new value (temp);
    $4->free ();
    delete $4;
    $8->free ();
    delete $8;
    delete[] $12;
  }				// end if
  else {
    str_flag = 1;
    str = new char[strlen ($4->str) + 8 + 3];
    strcpy (str, "_bo_ol_=");
    strcat (str, $4->str);
    strcat (str, "; $");
    yyrestart (yyin);
    yyparse ();
    value temp (syms->lookup ("_bo_ol_").get_value ());
    value one (1);
    while (_and (&temp, &one, err)->num) {
      str_flag = 1;
      str = new char[strlen ($8->str) + 2];
      strcpy (str, $8->str);
      strcat (str, " $");
      yyrestart (yyin);
      yyparse ();
      str_flag = 1;
      str = new char[strlen ($4->str) + 8 + 3];
      strcpy (str, "_bo_ol_=");
      strcat (str, $4->str);
      strcat (str, "; $");
      yyrestart (yyin);
      yyparse ();
      temp = (syms->lookup ("_bo_ol_").get_value ());
    }				// end while
    str_flag = 1;
    str = new char[strlen ($12)];
    strcpy (str, $12);
    yyrestart (yyin);
    $$ = new value ();
    $4->free ();
    delete $4;
    $8->free ();
    delete $8;
    delete[] $12;
  }				// end else
}

;

if_statement: IF '(' expression subroutin_inc ')' statement subroutin_dec END {
  // Begin catching if statement body
  my_yybegin(CATCH_MODE) ;   
} CATCH {

  if (text_flag) {
    char *temp = new char[2 + 1 + strlen ($3->str) + 1 +
			  strlen ($6->str) + 3 + strlen ($10)];
    strcpy (temp, "if(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    strcat (temp, $6->str);
    strcat (temp, "end");
    strcat (temp, $10);
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $6->free ();
    delete $6;
    delete[] $10;
  }				// end if
  else {
    value one (1);
    if (_and ($3, &one, err)->num) {
      str_flag = 1;
      str = new char[strlen ($6->str) + 2];
      strcpy (str, $6->str);
      strcat (str, " $");
      yyrestart (yyin);
      yyparse ();
    }				// end if
    str_flag = 1;
    str = new char[strlen ($10)];
    strcpy (str, $10);
    yyrestart (yyin);
    $$ = new value ();
    $3->free ();
    delete $3;
    $6->free ();
    delete $6;
    delete[] $10;
  }				// end else
}

| IF '(' expression subroutin_inc ')' statement subroutin_dec ELSE subroutin_inc statement subroutin_dec END {
  // Begin catching if statement body
  my_yybegin(CATCH_MODE);   
} CATCH {

  if (text_flag) {
    char *temp = new char[2 + 1 + strlen ($3->str) + 1 +
			  strlen ($6->str) + 4 + strlen ($10->str) + 3 +
			  strlen ($14)];
    strcpy (temp, "if(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    strcat (temp, $6->str);
    strcat (temp, "else");
    strcat (temp, $10->str);
    strcat (temp, "end");
    strcat (temp, $14);
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $6->free ();
    delete $6;
    $10->free ();
    delete $10;
    delete[] $14;
  }				// end if
  else {
    value one (1);
    if (_and ($3, &one, err)->num) {
      str_flag = 1;
      str = new char[strlen ($6->str) + 2];
      strcpy (str, $6->str);
      strcat (str, " $");
      yyrestart (yyin);
      yyparse ();
    }				// end if
    else {
      str_flag = 1;
      str = new char[strlen ($10->str) + 2];
      strcpy (str, $10->str);
      strcat (str, " $");
      yyrestart (yyin);
      yyparse ();
    }
    str_flag = 1;
    str = new char[strlen ($14)];
    strcpy (str, $14);
    yyrestart (yyin);
    $$ = new value ();
    $3->free ();
    delete $3;
    $6->free ();
    delete $6;
    $10->free ();
    delete $10;
    delete[] $14;
  }				// end else
}

;


for_statement: FOR '(' NAME ASSIGN expression COLON expression { 
  text_flag += 1; 
} ')' statement { 
  text_flag -= 1; 
} END {   
  // Start catching for statement body
  my_yybegin(CATCH_MODE);   
} CATCH {

  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3) + 1 +
			  strlen ($5->str) + 1 +
			  strlen ($7->str) + 1 +
			  strlen ($10->str) + 3 + strlen ($14)];
    strcpy (temp, "for(");
    strcat (temp, $3);
    strcat (temp, "=");
    strcat (temp, $5->str);
    strcat (temp, ":");
    strcat (temp, $7->str);
    strcat (temp, ")");
    strcat (temp, $10->str);
    strcat (temp, "end");
    strcat (temp, $14);
    $$ = new value (temp);
    delete[] $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $10->free ();
    delete $10;
    delete[] $14;
  }				// end if
  else {
    syms->insert (entry (strdup ($3), *$5));
    value temp (syms->lookup ($3).get_value ());
    while (le (&temp, $7, err)->num != (0)) {
      str_flag = 1;
      str = new char[strlen ($10->str) + 2];
      strcpy (str, $10->str);
      strcat (str, " $");
      yyrestart (yyin);
      yyparse ();
      value vone (1);
      temp = (syms->lookup ($3).get_value ());
      temp = *plus (&temp, &vone, err);
      syms->insert (entry (strdup ($3), temp));
    }				// end while
    str_flag = 1;
    str = new char[strlen ($14)];
    strcpy (str, $14);
    yyrestart (yyin);
    $$ = new value ();
    delete[] $3;
    delete $5;
    $7->free ();
    delete $7;
    $10->free ();
    delete $10;
    delete[] $14;
  }				// end else
}

| FOR '(' NAME ASSIGN expression COLON expression COLON expression {
  text_flag += 1;
} ')' statement {
  text_flag -= 1;
} END {
  // Start catching for statement body
  my_yybegin(CATCH_MODE);
} CATCH {

  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3) + 1 +
			  strlen ($5->str) + 1 +
			  strlen ($7->str) + 1 +
			  strlen ($9->str) + 1 +
			  strlen ($12->str) + 3 + strlen ($16)];
    strcpy (temp, "for(");
    strcat (temp, $3);
    strcat (temp, "=");
    strcat (temp, $5->str);
    strcat (temp, ":");
    strcat (temp, $7->str);
    strcat (temp, ":");
    strcat (temp, $9->str);
    strcat (temp, ")");
    strcat (temp, $12->str);
    strcat (temp, "end");
    strcat (temp, $16);
    $$ = new value (temp);
    delete[] $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    $12->free ();
    delete $12;
    delete[] $16;
  }				// end if
  else {
    syms->insert (entry (strdup ($3), *$5));
    value temp (syms->lookup ($3).get_value ());
    while (le (&temp, $9, err)->num != (0)) {
      str_flag = 1;
      str = new char[strlen ($12->str) + 2];
      strcpy (str, $12->str);
      strcat (str, " $");
      yyrestart (yyin);
      yyparse ();
      temp = (syms->lookup ($3).get_value ());
      temp = *plus (&temp, $7, err);
      syms->insert (entry (strdup ($3), temp));
    }				// end while
    str_flag = 1;
    str = new char[strlen ($16)];
    strcpy (str, $16);
    yyrestart (yyin);
    $$ = new value ();
    delete[] $3;
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    $12->free ();
    delete $12;
    delete[] $16;
  }				// end else
}

;

 // NAME = $3

expression: NAME ASSIGN expression {
  if (text_flag) {
    char *temp = new char[strlen ($1) + 1 + strlen ($3->str)];
    strcpy (temp, $1);
    strcat (temp, "=");
    strcat (temp, $3->str);
    $$ = new value (temp);
    delete[] $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type != FUNCT)
      syms->insert (entry ($1, *$3));
    if ($3->type == NUM)
      $3->num = ($3->num);
    else if ($3->type == DENSE)
      $3->dmat = new dense (*$3->dmat);
    else if ($3->type == SPARSE)
      $3->smat = new sparse (*$3->smat);
    else if ($3->type == STRING)
      $3->str = strdup ($3->str);
    else {
      sp_error ("Error, invalid type for operator assign.\n");
      YYERROR;
    }				// end else
    $$ = $3;
  }				// end else
}

 // NAME(x,y) = $6 
 // Set element $3 in matrix $1 to $6

| NAME '(' comma_list ')' ASSIGN expression {
  if (text_flag) {
    $3->init_curr_prev ();
    int len = strlen ($3->get_current ().get_value ().str) + 1;
    while (!$3->end ()) {
      $3->get_next ();
      len += strlen ($3->get_current ().get_value ().str) + 1;
    }				// end while
    char *temp = new char[strlen ($1) + 1 + len + 1 + 1 + strlen ($6->str)];
    $3->init_curr_prev ();
    strcpy (temp, $1);
    strcat (temp, "(");
    strcat (temp, $3->get_current ().get_value ().str);
    while (!$3->end ()) {
      $3->get_next ();
      strcat (temp, ",");
      strcat (temp, $3->get_current ().get_value ().str);
    }				// end while
    strcat (temp, ")=");
    strcat (temp, $6->str);
    $$ = new value (temp);
    $3->remove_all ();
    delete $3;
    delete[] $1;
    $6->free ();
    delete $6;
  }				// end if
  else {
    err = 0;
    if (!syms->contains ($1)) {
      $3->remove_all ();
      delete[] $1;
      delete $3;
      $6->free ();
      delete $6;
      sp_error("Error, \"");
      sp_error($1);
      sp_error("\" is not a defined matrix\n");
      YYERROR;
    }				// end if
    int type = syms->lookup ($1).get_value ().type;
    if (type == DENSE || type == SPARSE)
      $$ = set_element ($1, $3, $6, err);
    delete[] $1;
    delete $3;
    $6->free ();
    delete $6;
    if (err)
      YYERROR;
  }				// end else
}

 // NAME($3:$5)
 // Range of vector NAME from $3 to $5 

| NAME '(' expression COLON expression ')' {
  if(text_flag) {
    char * temp = new char[strlen($1) + 1 + strlen($3->str) + 1 + 
			  strlen($5->str) + 1];
    sprintf(temp, "%s(%s:%s)", $1, $3->str, $5->str);
    delete [] $1;
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }
  else {
    int err = 0;
    $$ = get_range ($1, $3, new value(1), $5, err);
    delete[] $1;
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if(err) {
      YYERROR;
    }
  }
}

 // NAME($3:$5:$7)
 // Range of vector NAME from $3 to $7 with step $5

| NAME '(' expression COLON expression COLON expression ')' {
  if(text_flag) {
    char * temp = new char[strlen($1) + 1 + strlen($3->str) + 1 + 
			  strlen($5->str) + 1 + strlen($7->str) + 1];
    sprintf(temp, "%s(%s:%s:%s)", $1, $3->str, $5->str, $7->str);
    delete [] $1;
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
  }
  else {
    int err = 0;
    $$ = get_range ($1, $3, $5, $7, err);
    delete[] $1;
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    if(err) {
      YYERROR;
    }
  }
}

 // $1 || $3

| expression OR expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "||");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = _or ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 && $3

| expression AND expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "&&");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = _and ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end  else
}

 // $1 < $3

| expression LT expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "<");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = lt ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 <= $3

| expression LE expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "<=");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = le ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 == $3

| expression EQ expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "==");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = eq ($1, $3, err);
    delete $1;
    $3->free ();
    delete $3;
    $1->free ();
    if (err)
      YYERROR;
  }				// end else
}

 // $1 >= $3

| expression GE expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ">=");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = ge ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 > $3

| expression GT expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ">");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = gt ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 != $3

| expression NE expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "!=");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = ne ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 + $3

| expression PLUS expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "+");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = plus ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 - $3

| expression MINUS expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "-");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = minus ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 * $3

| expression MULT expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "*");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = mult ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 .* $3

| expression EMULT expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ".*");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = emult ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 / $3

| expression DIV expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "/");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = div ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 ./ $3

| expression EDIV expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "./");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = ediv ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 ' $3
 // or
 // $1'

| expression LDIV expression | expression ELDIV expression | expression TRANSP {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1];
    strcpy (temp, $1->str);
    strcat (temp, "'");
    $$ = new value (temp);
    $1->free ();
    delete $1;
  }				// end if
  else {
    err = 0;
    $$ = transp ($1, err);
    $1->free ();
    delete $1;
    if (err)
      YYERROR;
  }				// end else
}

 // +$2
 // (Unary plus does nothing)

| PLUS expression %prec UPLUS {
  if (text_flag)
    $$ = $2;
  else
    $$ = $2;
}

 // -$2

| MINUS expression %prec UMINUS {
  if (text_flag) {
    char *temp = new char[1 + strlen ($2->str)];
    strcpy (temp, "-");
    strcat (temp, $2->str);
    $$ = new value (temp);
    $2->free ();
    delete $2;
  }				// end if
  else {
    err = 0;
    $$ = uminus ($2, err);
    $2->free ();
    delete $2;
    if (err)
      YYERROR;
  }				// end else
}

 // ++NAME

| INC NAME {
  if (text_flag) {
    char *temp = new char[2 + strlen ($2)];
    strcpy (temp, "++");
    strcat (temp, $2);
    $$ = new value (temp);
    delete[] $2;
  }				// end if
  else {
    if (syms->contains ($2)) {
      value temp (syms->lookup ($2).get_value ());
      value tempvn (1);
      err = 0;
      $$ = plus (&temp, &tempvn, err);
      if (err)
	YYERROR;
      syms->insert (entry ($2, *plus (&temp, &tempvn, err)));
    }				// end if
    else {
      sp_error("Error, undefined name: \"");
      sp_error($2);
      sp_error("\"\n");
      YYERROR;
    }				// end else
  }				// end else
}

 // NAME++

| NAME INC {
  if (text_flag) {
    char *temp = new char[2 + strlen ($1)];
    strcpy (temp, $1);
    strcat (temp, "++");
    $$ = new value (temp);
    delete[] $1;
  }				// end if
  else {
    if (syms->contains ($1)) {
      value temp (syms->lookup ($1).get_value ());
      value tempvn (1);
      $$ = new value (syms->lookup ($1).get_value ());
      if ($$->type == NUM)
	$$->num = ($$->num);
      else if ($$->type == DENSE)
	$$->dmat = new dense (*$$->dmat);
      else if ($$->type == SPARSE)
	$$->smat = new sparse (*$$->smat);
      else if ($$->type == STRING)
	$$->str = strdup ($$->str);
      else {
	sp_error ("Error, invalid type for operator ++\n");
	YYERROR;
      }				// end else
      err = 0;
      syms->insert (entry ($1, *plus (&temp, &tempvn, err)));
      if (err)
	YYERROR;

    }				// end if
    else {
      sp_error("Error, undefined name: \"");
      sp_error($1);
      sp_error("\"\n");
      YYERROR;
    }				// end else
  }				// end else           
}

 // --NAME

| DEC NAME {
  if (text_flag) {
    char *temp = new char[strlen ($2) + 2];
    strcpy (temp, "--");
    strcat (temp, $2);
    $$ = new value (temp);
    delete[] $2;
  }				// end if
  else {
    if (syms->contains ($2)) {
      value temp (syms->lookup ($2).get_value ());
      value tempvn (1);
      err = 0;
      $$ = minus (&temp, &tempvn, err);
      if (err)
	YYERROR;
      syms->insert (entry ($2, *minus (&temp, &tempvn, err)));
    }				// end if
    else {
      sp_error("Error, undefined name: \"");
      sp_error($2);
      sp_error("\"\n");
      YYERROR;
    }				// end else
  }				// end else
}

 // NAME--

| NAME DEC {
  if (text_flag) {
    char *temp = new char[strlen ($1) + 2];
    strcpy (temp, $1);
    strcpy (temp, "--");
    $$ = new value (temp);
    delete[] $1;
  }				// end if
  else {
    if (syms->contains ($1)) {
      value temp (syms->lookup ($1).get_value ());
      value tempvn (1);
      $$ = new value (syms->lookup ($1).get_value ());
      if ($$->type == NUM)
	$$->num = ($$->num);
      else if ($$->type == DENSE)
	$$->dmat = new dense (*$$->dmat);
      else if ($$->type == SPARSE)
	$$->smat = new sparse (*$$->smat);
      else if ($$->type == STRING)
	$$->str = strdup ($$->str);
      else {
	sp_error ("Error, invalid type for operator --\n");
	YYERROR;
      }				// end else
      err = 0;
      syms->insert (entry ($1, *minus (&temp, &tempvn, err)));
      if (err)
	YYERROR;

    }				// end if
    else {
      sp_error("Error, undefined name: \"");
      sp_error($1);
      sp_error("\"\n");
      YYERROR;
    }				// end else
  }				// end else           
}

 // !$2

| NOT expression {
  if (text_flag) {
    char *temp = new char[1 + strlen ($2->str)];
    strcpy (temp, "!");
    strcat (temp, $2->str);
    $$ = new value (temp);
    delete[] $2;
  }				// end if
  else {
    err = 0;
    $$ = _not ($2, err);
    $2->free ();
    delete $2;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 ^ $3

| expression POW expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 1 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, "^");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = pow ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // $1 .^ $3

| expression EPOW expression {
  if (text_flag) {
    char *temp = new char[strlen ($1->str) + 2 + strlen ($3->str)];
    strcpy (temp, $1->str);
    strcat (temp, ".^");
    strcat (temp, $3->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = epow ($1, $3, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

 // ( $2 )
 // Sp that parenthesized expressions get evaluated first

| '(' expression ')' {
  if (text_flag) {
    char *temp = new char[1 + strlen ($2->str) + 1];
    strcpy (temp, "(");
    strcat (temp, $2->str);
    strcat (temp, "(");
    $$ = new value (temp);
    $2->free ();
    delete $2;
  }				// end if
  else
    $$ = $2;
}

 // NAME

| NAME {
  if (text_flag)
    $$ = new value ($1);
  else {
    if (syms->contains ($1)) {
      $$ = new value (syms->lookup ($1).get_value ());
      if ($$->type == NUM)
	$$->num = ($$->num);
      else if ($$->type == DENSE)
	$$->dmat = new dense (*$$->dmat);
      else if ($$->type == SPARSE)
	$$->smat = new sparse (*$$->smat);
      else if ($$->type == STRING)
	$$->str = strdup ($$->str);
      else {
	sp_error ("Error, function name must be followed by arg list.\n");
	YYERROR;
      }				// end else
    }				// end if
    else {
      sp_error("Error, undefined name: \"");
      sp_error($1);
      sp_error("\"\n");
      YYERROR;
    }				// end else
  }				// end else  
}

| NUMBER {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

| STR {
  if (text_flag)
    $$ = $1;
  else {
    $$ = $1;
    unsigned int i;
    // Get rid of the quotation marks
    for (i = 0; i <= strlen ($$->str); i += 1)
      *($$->str + i) = *($$->str + i + 1);
    *($$->str + strlen ($$->str) - 1) = '\0';
  }				// end else
}

| dense {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

| function_call {
  if (text_flag)
    $$ = $1;
  else
    $$ = $1;
}

;


function_call: NAME '(' comma_list ')' {
  // Send lex into "catch-all" mode
  my_yybegin(CATCH_MODE);   

  if (yychar == 0) {
    str_flag = 1;
    str = strdup ("$");
  }				// end if
  lk = yychar;
  yyclearin;
} CATCH {
  int look = lk;
  if (text_flag) {
    $3->init_curr_prev ();
    int len = strlen ($3->get_current ().get_value ().str) + 1;
    while (!$3->end ()) {
      $3->get_next ();
      len += strlen ($3->get_current ().get_value ().str) + 1;
    }				// end while
    char *temp = new char[strlen ($1) + 1 + len + 1];
    $3->init_curr_prev ();
    strcpy (temp, $1);
    strcat (temp, "(");
    strcat (temp, $3->get_current ().get_value ().str);
    while (!$3->end ()) {
      $3->get_next ();
      strcat (temp, ",");
      strcat (temp, $3->get_current ().get_value ().str);
    }				// end while
    strcat (temp, ")");
    yychar = look;
    $$ = new value (temp);
    $3->remove_all ();
    delete $3;
    delete[] $1;
    delete[] $6;
  }				// end if
  else {
    err = 0;
    if (syms->contains ($1)) {
      int type = syms->lookup ($1).get_value ().type;
      if (type == DENSE || type == SPARSE)
	$$ = get_element ($1, $3, err);
      else if (root.contains ($1)) {
	int type = root.lookup ($1).get_value ().type;
	if (type == FUNCT)
	  $$ = eval_funct ($1, $3, err);
      }				// end if
    }				// end if
    else {
      if (root.contains ($1)) {
	int type = root.lookup ($1).get_value ().type;
	if (type == FUNCT)
	  $$ = eval_funct ($1, $3, err);
      }				// end if
      else {
	$3->remove_all ();
	sp_error("Error, \"");
	sp_error($1);
	sp_error("\" is not defined\n");
	delete[] $1;
	delete $3;
	YYERROR;
      }				// end else
    }				// end else
    delete[] $1;
    delete $3;
    if (err) {
      delete[] $6;
      YYERROR;
    }				// end if
    char *temp = new char[8 + strlen ($6)];
    if (look == SEMI)
      strcpy (temp, ";");
    if (look == COMMA)
      strcpy (temp, ",");
    if (look == ASSIGN)
      strcpy (temp, "=");
    if (look == OR)
      strcpy (temp, "||");
    if (look == AND)
      strcpy (temp, "&&");
    if (look == LT)
      strcpy (temp, "<");
    if (look == LE)
      strcpy (temp, "<=");
    if (look == EQ)
      strcpy (temp, "==");
    if (look == GE)
      strcpy (temp, ">=");
    if (look == GT)
      strcpy (temp, ">");
    if (look == NE)
      strcpy (temp, "!=");
    if (look == COLON)
      strcpy (temp, ":");
    if (look == PLUS)
      strcpy (temp, "+");
    if (look == MINUS)
      strcpy (temp, "-");
    if (look == MULT)
      strcpy (temp, "*");
    if (look == EMULT)
      strcpy (temp, ".*");
    if (look == DIV)
      strcpy (temp, "/");
    if (look == EDIV)
      strcpy (temp, "./");
    if (look == LDIV)
      strcpy (temp, "\\");
    if (look == ELDIV)
      strcpy (temp, ".\\");
    if (look == TRANSP)
      strcpy (temp, "'");
    if (look == UPLUS)
      strcpy (temp, "+");
    if (look == UMINUS)
      strcpy (temp, "-");
    if (look == INC)
      strcpy (temp, "++");
    if (look == DEC)
      strcpy (temp, "--");
    if (look == NOT)
      strcpy (temp, "!");
    if (look == POW)
      strcpy (temp, "^");
    if (look == EPOW)
      strcpy (temp, ".^");
    if (look == BREAK)
      strcpy (temp, "break");
    if (look == int ('('))
      strcpy (temp, "(");
    if (look == int (')'))
      strcpy (temp, ")");
    if (look == CONTINUE)
      strcpy (temp, "continue");
    if (look == ELSEIF)
      strcpy (temp, "elseif");
    if (look == END)
      strcpy (temp, "end");
    if (look == FOR)
      strcpy (temp, "for");
    if (look == FUNCTION)
      strcpy (temp, "function");
    if (look == GLOBAL)
      strcpy (temp, "global");
    if (look == IF)
      strcpy (temp, "if");
    if (look == RETURN)
      strcpy (temp, "return");
    if (look == WHILE)
      strcpy (temp, "while");
    strcat (temp, $6);
    str_flag = 1;
    str = temp;
    delete[] $6;
    yyrestart (yyin);
  }				// end else
}

| NAME '(' ')' { 
  // Send lex into "catch-all" mode where it reads everything
  // up the next dollar sign and then returns it as a string
  // (without tokenizing it).
  my_yybegin(CATCH_MODE);   
  //yy_start = 1 + 2 * 1 
} CATCH {
  if (text_flag) {
    char *temp = new char[strlen ($1) + 1 + 1 + strlen ($5)];
    strcpy (temp, $1);
    strcat (temp, "()");
    strcat (temp, $5);
    $$ = new value (temp);
    delete[] $1;
    delete[] $5;
  }				// end if
  else {
    err = 0;
    if (syms->contains ($1)) {
      //      int type = syms->lookup ($1).get_value ().type;
      if (root.contains ($1)) {
	int type = root.lookup ($1).get_value ().type;
	if (type == FUNCT)
	  $$ = eval_funct ($1, new list (), err);
      }				// end if
    }				// end if
    else {
      if (root.contains ($1)) {
	int type = root.lookup ($1).get_value ().type;
	if (type == FUNCT)
	  $$ = eval_funct ($1, new list (), err);
      }				// end if
      else {
	sp_error("Error, \"");
	sp_error($1);
	sp_error("\" is not defined\n");
	delete[] $1;
	delete[] $5;
	YYERROR;
      }				// end else
    }				// end else
    delete[] $1;
    if (err) {
      delete[] $5;
      YYERROR;
    }				// end if
    str_flag = 1;
    str = $5;
    yyrestart (yyin);
  }				// end else

}

| LOAD '(' expression ')' { 
  // Send lex into "catch-all" mode where it reads everything
  // up the next dollar sign and then returns it as a string
  // (without tokenizing it).
  my_yybegin(CATCH_MODE);   
  //  yy_start = 1 + 2 * 1 
} CATCH {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1 + strlen ($6)];
    strcpy (temp, "load(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    strcat (temp, $6);
    $$ = new value (temp);
    $3->free ();
    delete $3;
    delete[] $6;
  }				// end if
  else {
    err = 0;
    $$ = load ($3, 1, err);
    $3->free ();
    delete $3;
    if (err) {
      delete[] $6;
      YYERROR;
    }				// end if
    str_flag = 1;
    str = $6;
    yyrestart (yyin);
  }				// end else
}

| DEMO '(' expression ')' { 
  // Send lex into "catch-all" mode where it reads everything
  // up the next dollar sign and then returns it as a string
  // (without tokenizing it).
  my_yybegin(CATCH_MODE);   
  //  yy_start = 1 + 2 * 1 
} CATCH {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1 + strlen ($6)];
    strcpy (temp, "demo(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    strcat (temp, $6);
    $$ = new value (temp);
    $3->free ();
    delete $3;
    delete[] $6;
  }				// end if
  else {
    err = 0;
    $$ = load ($3, 1, err);
    $3->free ();
    delete $3;
    if (err) {
      delete[] $6;
      YYERROR;
    }				// end if
    str_flag = 1;
    str = $6;
    yyrestart (yyin);
  }				// end else
}

| ALIAS NAME ASSIGN expression {
  if (text_flag) {
    char *temp = new char[6 + strlen ($2) + 1 + strlen ($4->str)];
    strcpy (temp, "alias ");
    strcat (temp, $2);
    strcat (temp, "=");
    strcat (temp, $4->str);
    $$ = new value (temp);
    delete[] $2;
    $4->free ();
    delete $4;
  }				// end if
  else {
    if ($4->type == STRING) {
      ali->insert (entry ($2, *$4));
      $4->str = strdup ($4->str);
    }
    else {
      sp_error("Error, invalid type for operator assign.\n");
      YYERROR;
    }				// end else
    $$ = $4;
  }				// end else
}


| HELP { 
  // Send lex into "catch-all" mode where it reads everything
  // up the next dollar sign and then returns it as a string
  // (without tokenizing it).
  //  cout << "Entering catch mode" << endl;
  my_yybegin(CATCH_MODE);   
  //  yy_start = 1 + 2 * 1 
} CATCH {
  assert ($3[strlen ($3) - 1] == '$');
  $3[strlen ($3) - 1] = '\0';
  help ($3);
  str_flag = 1;
  str = strdup ("$");
  $$ = new value ();
}

| NROW '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "nrow(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == DENSE)
      $$ = new value ($3->dmat->get_nrow ());
    else if ($3->type == SPARSE)
      $$ = new value ($3->smat->get_nrow ());
    else
      $$ = new value (0);
    $3->free ();
    delete $3;
  }				// end else
}

| NCOL '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "ncol(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == DENSE)
      $$ = new value ($3->dmat->get_ncol ());
    else if ($3->type == SPARSE)
      $$ = new value ($3->smat->get_ncol ());
    else
      $$ = new value (0);
    $3->free ();
    delete $3;
  }				// end else
}

| NNZ '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "nnz(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == DENSE)
      $$ = new value ($3->dmat->get_nnz ());
    else if ($3->type == SPARSE)
      $$ = new value ($3->smat->get_nnz ());
    else
      $$ = new value (0);
    $3->free ();
    delete $3;
  }				// end else
}

| SIZE2 '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "size(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == DENSE) {
      if ($3->dmat->get_nrow () == 1) {
	$$ = new value ($3->dmat->get_ncol ());
      }
      else {
	dense *dtemp = new dense (1, 2);
	dtemp->set_element (1, 1, $3->dmat->get_nrow ());
	dtemp->set_element (1, 2, $3->dmat->get_ncol ());
	$$ = new value (dtemp);
      }
    }
    if ($3->type == SPARSE) {
      if ($3->smat->get_nrow () == 1) {
	$$ = new value ($3->smat->get_ncol ());
      }
      else {
	dense *temp = new dense (1, 2);
	temp->set_element (1, 1, $3->smat->get_nrow ());
	temp->set_element (1, 2, $3->smat->get_ncol ());
	$$ = new value (temp);
      }
    }
    $3->free ();
    delete $3;
  }				// end else
}

| EYE '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "eye(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = eye ($3, err);
    $3->free ();
    delete $3;
    if (err) {
      sp_error("Error while creating identity matrix\n");
      YYERROR;
    }				// end if
  }				// end else
}

| SPZEROS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[8 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "spzeroes(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = spzeroes ($3, err);
    $3->free ();
    delete $3;
    if (err) {
      YYERROR;
    }				// end if
  }				// end else
}

| SPZEROS '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[8 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "spzeroes(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = spzeroes ($3, $5, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err) {
      YYERROR;
    }				// end if
  }				// end else
}

| ZEROS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "zeroes(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = zeroes ($3, err);
    $3->free ();
    delete $3;
    if (err) {
      YYERROR;
    }				// end if
  }				// end else
}

| ZEROS '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "zeroes(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = zeroes ($3, $5, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err) {
      YYERROR;
    }				// end if
  }				// end else
}

| ONES '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "ones(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = ones ($3, err);
    $3->free ();
    delete $3;
    if (err) {
      YYERROR;
    }				// end if
  }				// end else
}

| ONES '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[4 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "ones(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = ones ($3, $5, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err) {
      YYERROR;
    }				// end if
  }				// end else
}

| INIT '(' expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[8 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1];
    strcpy (temp, "constmat(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
  }				// end if
  else {
    err = 0;
    $$ = init ($3, $5, $7, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    if (err)
      YYERROR;
  }				// end else
}

| CLEAR '(' NAME ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3) + 1];
    strcpy (temp, "clear(");
    strcat (temp, $3);
    strcat (temp, ")");
    $$ = new value (temp);
    delete[] $3;
  }				// end if
  else {
    if (syms->contains ($3))
      syms->remove ($3);
    if (ali->contains ($3)) {
      ali->remove ($3);
    }
    clear_alias = 0;
  }
}

| CLEAR NAME {
  if (text_flag) {
    char *temp = new char[5 + strlen ($2)];
    strcpy (temp, "clear(");
    strcat (temp, $2);
    strcat (temp, ")");
    $$ = new value (temp);
    delete[] $2;
  }				// end if
  else {
    if (syms->contains ($2))
      syms->remove ($2);
    if (ali->contains ($2)) {
      ali->remove ($2);
    }
    clear_alias = 0;
  }
}

| READMT '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "hbin(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = readmt ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

/*========================================================================
				SUBMAT
========================================================================*/

| SUBMAT '(' expression COMMA expression COMMA expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1 + strlen ($9->str) + 1 +
	       strlen ($11->str) + 1];
    strcpy (temp, "submat(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ",");
    strcat (temp, $9->str);
    strcat (temp, ",");
    strcat (temp, $11->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    $11->free ();
    delete $11;
  }				// end if
  else {
    err = 0;
    $$ = submat ($3, $5, $7, $9, $11, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    $11->free ();
    delete $11;
    if (err)
      YYERROR;
  }				// end else

}

| EXTRACT '(' expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[7 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1];
    strcpy (temp, "extract(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
  }				// end if
  else {
    err = 0;
    if ($3->type == DENSE) {
      if ($3->dmat->get_nrow () == 1)
	$$ = submat (new value (1), new value (1), $5, $7, $3, err);
    }
    else {
      yyerror ("Error, the first argument must be a vector");
      err = 1;
    }
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    if (err)
      YYERROR;
  }				// end else
}

| EXTRACT_ROW '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[11 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "extract_row(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    if ($3->type == DENSE)
      $$ =
	submat ($5, $5, new value (1), new value ($3->dmat->get_ncol ()), $3,
		err);
    if ($3->type == SPARSE)
      $$ =
	submat ($5, $5, new value (1), new value ($3->smat->get_ncol ()), $3,
		err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| EXTRACT_COL '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[11 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "extract_col(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    if ($3->type == DENSE)
      $$ =
	submat (new value (1), new value ($3->dmat->get_nrow ()), $5, $5, $3,
		err);
    if ($3->type == SPARSE)
      $$ =
	submat (new value (1), new value ($3->smat->get_nrow ()), $5, $5, $3,
		err);
    if ($3->type != DENSE && $3->type != SPARSE) {
      yyerror
	("Error, the 1st argument must be either a sparse or a dense matrix. ");
      err = 1;
    }
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| FILTER '(' expression COMMA expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1 + strlen ($9->str) + 1];
    strcpy (temp, "filter(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ",");
    strcat (temp, $9->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
  }				// end if
  else {
    err = 0;
    $$ = filter ($3, $5, $7, $9, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;

    if (err)
      YYERROR;
  }				// end else

}

| CSORT '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "csort(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = csort ($3, $5, err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| RMDIA '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "rmdia(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = rmdia ($3, $5, err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| GETDIA '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "getdia(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = getdia ($3, $5, err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| GETU '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "getu(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = getu ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| GETL '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "getl(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = getl ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| GETBWD '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "getbwd(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = getbwd ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| GETBWDU '(' expression ')' {
  if (text_flag) {
    char *temp = new char[7 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "getbwdu(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = getbwdu ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| GETBWDL '(' expression ')' {
  if (text_flag) {
    char *temp = new char[7 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "getbwdl(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = getbwdl ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| BLKFND '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "blkfnd(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = blkfnd ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}


| CNRMS '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "cnrms(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = cnrms ($3, $5, err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| COSCAL '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "coscal(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = coscal ($3, $5, err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| ROSCAL '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "roscal(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = roscal ($3, $5, err);
    $3->free ();
    delete[] $3;
    $5->free ();
    delete $5;

    if (err)
      YYERROR;
  }				// end else
}

| GET1UP '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "get1up(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = get1up ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| ADDBLK '(' expression COMMA expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1 + strlen ($9->str) + 1];
    strcpy (temp, "addblk(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ",");
    strcat (temp, $9->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
  }				// end if
  else {
    err = 0;
    $$ = addblk ($3, $5, $7, $9, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;

    if (err)
      YYERROR;
  }				// end else

}

/*=======================================================================
			GEN57PT
========================================================================*/

| GEN57PT '(' expression COMMA expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1 + strlen ($9->str) + 1];
    strcpy (temp, "fdmat(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ",");
    strcat (temp, $9->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
  }				// end if
  else {
    err = 0;
    $$ = gen57pt ($3, $5, $7, $9, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    if (err)
      YYERROR;
  }				// end else
}


| SOLVE_SYSTEM '(' expression COMMA expression COMMA expression COMMA expression COMMA expression COMMA expression COMMA expression COMMA expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[6 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1 + strlen ($9->str) + 1 +
	       strlen ($11->str) + 1 + strlen ($13->str) + 1 +
	       strlen ($15->str) + 1 + strlen ($17->str) + 1 +
	       strlen ($19->str) + 1 + strlen ($21->str) + 1];

    strcpy (temp, "fdmat(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ",");
    strcat (temp, $9->str);
    strcat (temp, ",");
    strcat (temp, $11->str);
    strcat (temp, ",");
    strcat (temp, $13->str);
    strcat (temp, ",");
    strcat (temp, $15->str);
    strcat (temp, ",");
    strcat (temp, $17->str);
    strcat (temp, ",");
    strcat (temp, $19->str);
    strcat (temp, ",");
    strcat (temp, $21->str);
    strcat (temp, ")");

    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    $11->free ();
    delete $11;
    $13->free ();
    delete $13;
    $15->free ();
    delete $15;
    $17->free ();
    delete $17;
    $19->free ();
    delete $19;
    $21->free ();
    delete $21;
  }				// end if
  else {
    err = 0;
    $$ =
      solve_system ($3, $5, $7, $9, $11, $13, $15, $17, $19, $21, err);

    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    $11->free ();
    delete $11;
    $13->free ();
    delete $13;
    $15->free ();
    delete $15;
    $17->free ();
    delete $17;
    $19->free ();
    delete $19;
    $21->free ();
    delete $21;
    if (err)
      YYERROR;
  }				// end else
}

| SOLVE_SYSTEM '(' expression COMMA expression COMMA expression ')' {

  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	      strlen ($7->str) + 1];

    strcpy (temp, "fdmat(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ")");

    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
  }				// end if
  else {
    err = 0;
    $$ = solve_system ($3, $5, $7, err);

    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    if (err)
      YYERROR;
  }				// end else
}

| MULTIC '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "multic(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = multic ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| COLORS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "colors(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = colors ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| COLORSET '(' expression COMMA NAME COMMA NAME ')' {
  if (text_flag) {
    char *temp =
      new char[8 + 1 + strlen ($3->str) + 1 + strlen ($5) + 1 + strlen ($7) +
	       1];
    strcpy (temp, "colorset(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5);
    strcat (temp, ",");
    strcat (temp, $7);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    delete [] $5;
    delete [] $7;
  }
  else {
    err = 0;
    value *b = il_multic ($3, err);
    if (err) {
      $3->free();
      delete $3;
      delete [] $5;
      delete [] $7;
      YYERROR;
    }
    syms->insert (entry (strdup($5), *b));

    value *c = iord_multic ($3, err);
    if (err) {
      $3->free();
      delete $3;
      delete [] $5;
      delete [] $7;
      YYERROR;
    }
    syms->insert (entry (strdup($7), *c));

    several_init = 0;
    $$ = new value ();

    sp_out($5);
    sp_out(" now contains the pointers to the beginning of each color set\n");
    sp_out($7);
    sp_out(" now contains the permutation array\n");

    $3->free();
    delete $3;
    delete [] $5;
    delete [] $7;
    YYERROR;
  }
}

| INDSET0 '(' expression ')' {
  if (text_flag) {
    char *temp = new char[7 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "indset0(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = indset0 ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}

| INDSET1 '(' expression ')' {
  if (text_flag) {
    char *temp = new char[7 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "indset1(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = indset1 ($3, err);
    $3->free ();
    delete $3;
    if (err)
      YYERROR;
  }				// end else
}


| COLORS2 '(' expression ')' {
  if (text_flag) {
    char *temp = new char[7 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "colors2(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    int nco = 0;
    $$ = colors2 ($3, err, nco);
    $3->free ();
    delete $3;
    //    cout << "number of colors: " << nco << "\n";
    sp_out("number of colors: "); sp_out(nco); sp_out("\n");
    if (err)
      YYERROR;
  }				// end else
}



| PRTMT '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "hbout(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = prtmt ($3, $5, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err)
      YYERROR;
  }				// end else
}

| RPERM '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "rperm(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = rperm ($3, $5, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err)
      YYERROR;
  }				// end else
}

| CPERM '(' expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1];
    strcpy (temp, "cperm(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = cperm ($3, $5, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err)
      YYERROR;
  }				// end else
}

| DPERM '(' expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[5 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1];
    strcpy (temp, "dperm(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
  }				// end if
  else {
    err = 0;
    $$ = dperm ($3, $5, $7, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    if (err)
      YYERROR;
  }				// end else
}


| SIN '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "sin(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, sin, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function sin.");
      YYERROR;
    }				// end if
  }				// end else
}

| SQRT '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "sqrt(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, sqrt, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function sqrt.");
      YYERROR;
    }				// end if
  }				// end else
}

| COS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "cos(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, cos, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function cos.");
      YYERROR;
    }				// end if
  }				// end else
}

| TAN '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "tan(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, tan, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function tan.");
      YYERROR;
    }				// end if
  }				// end else
}

| COT '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "cot(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, cot, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function cot.");
      YYERROR;
    }				// end if
  }				// end else
}

| SC '(' expression ')' {
  if (text_flag) {
    char *temp = new char[2 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "sec(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, sec, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function sec.");
      YYERROR;
    }				// end if
  }				// end else
}

| CSC '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "csc(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, csc, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function csc.");
      YYERROR;
    }				// end if
  }				// end else
}

| ARCSIN '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "asin(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, asin, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function asin.");
      YYERROR;
    }				// end if
  }				// end else
}

| ARCCOS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "acos(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, acos, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function acos.");
      YYERROR;
    }				// end if
  }				// end else
}

| ARCTAN '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "atan(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, atan, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function atan.");
      YYERROR;
    }				// end if
  }				// end else
}

| ARCCOT '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "acot(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, acot, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function acot.");
      YYERROR;
    }				// end if
  }				// end else
}

| ARCSEC '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "asec(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, asec, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function asec.");
      YYERROR;
    }				// end if
  }				// end else
}

| ARCCSC '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "acsc(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, acsc, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function acsc.");
      YYERROR;
    }				// end if
  }				// end else
}

| EXP '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "exp(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, exp, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function exp.");
      YYERROR;
    }				// end if
  }				// end else
}

| LOG '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "log(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, log, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function log.");
      YYERROR;
    }				// end if
  }				// end else
}

| LOG10 '(' expression ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "log10(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, log10, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function log10.");
      YYERROR;
    }				// end if
  }				// end else
}

| ABS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[3 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "abs(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, fabs, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function abs.");
      YYERROR;
    }				// end if
  }				// end else
}

| CEIL '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "ceil(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, ceil, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function ceil.");
      YYERROR;
    }				// end if
  }				// end else
}

| FLOOR '(' expression ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "floor(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, floor, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function floor.");
      YYERROR;
    }				// end if
  }				// end else
}

| ROUND '(' expression ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "round(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, round, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function round.");
      YYERROR;
    }				// end if
  }				// end else
}

| TRUNC '(' expression ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "trunc(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = function ($3, trunc, err);
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function trunc.");
      YYERROR;
    }				// end if
  }				// end else
}

| RND '(' expression ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "rand(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    err = 0;
    $$ = new value (rnd0nb ((int) $3->num, 0.0));
    $3->free ();
    delete $3;
    if (err) {
      yyerror ("Error, invalid type for function rand.");
      YYERROR;
    }				// end if
  }				// end else
}

| RND '(' expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[4 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1];
    strcpy (temp, "rand(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
  }				// end if
  else {
    err = 0;
    $$ = rando ($3, $5, $7, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    if (err) {
      yyerror ("Error, invalid type for function rand.");
      YYERROR;
    }				// end if
  }				// end else
}

| RND '(' expression COMMA expression COMMA expression COMMA expression ')' {
  if (text_flag) {
    char *temp =
      new char[4 + 1 + strlen ($3->str) + 1 + strlen ($5->str) + 1 +
	       strlen ($7->str) + 1 + strlen ($9->str) + 1];
    strcpy (temp, "rand(");
    strcat (temp, $3->str);
    strcat (temp, ",");
    strcat (temp, $5->str);
    strcat (temp, ",");
    strcat (temp, $7->str);
    strcat (temp, ",");
    strcat (temp, $9->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
  }				// end if
  else {
    err = 0;
    $$ = rando ($3, $5, $7, $9, err);
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    $7->free ();
    delete $7;
    $9->free ();
    delete $9;
    if (err) {
      yyerror ("Error, invalid type for function rand.");
      YYERROR;
    }				// end if
  }				// end else
}


| RND '(' ')' {
  if (text_flag) {
    char *temp = new char[4 + 1 + 1];
    strcpy (temp, "rand()");
    $$ = new value (temp);
  }				// end if
  else
    $$ = new value (rnd (0.0));
}

| IRND '(' expression ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "irand(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
  }				// end if
  else
    $$ = new value (irnd ((int) $3->num, 0.0));
}


| DENSIFY '(' expression ')' {
  if (text_flag) {
    char *temp = new char[5 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "dense(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == DENSE)
      $$ = $3;
    else {
      if ($3->type == SPARSE) {
	$$ = new value (new dense (*$3->smat));
	$3->free ();
      }				// end if
      else {
	yyerror ("Error, invalid type for function dense.");
	YYERROR;
      }				// end else
    }				// end else
  }				// end else
}

| SPARSIFY '(' expression ')' {
  if (text_flag) {
    char *temp = new char[6 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "sparse(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == SPARSE)
      $$ = $3;
    else {
      if ($3->type == DENSE) {
	$$ =
	  new value (new
		     sparse ($3->dmat->get_nrow (), $3->dmat->get_ncol (),
			     $3->dmat->get_pointer ()));
	$3->free ();
      }				// end if
      else {
	yyerror ("Error, invalid type for function sparse.");
	YYERROR;
      }				// end else
    }				// end else
  }				// end else
}

| ISNUMBER '(' expression ')' {
  if (text_flag) {
    char *temp = new char[9 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "is_number(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == NUM)
      $$ = new value (1);
    else
      $$ = new value (0);
  }				// end else
}

| ISDENSE '(' expression ')' {
  if (text_flag) {
    char *temp = new char[8 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "is_dense(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == DENSE)
      $$ = new value (1);
    else
      $$ = new value (0);
  }				// end else
}

| ISSPARSE '(' expression ')' {
  if (text_flag) {
    char *temp = new char[9 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "is_sparse(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == SPARSE)
      $$ = new value (1);
    else
      $$ = new value (0);
  }				// end else
}

| ISSTRING '(' expression ')' {
  if (text_flag) {
    char *temp = new char[9 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "is_string(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == STRING)
      $$ = new value (1);
    else
      $$ = new value (0);
  }				// end else
}

| ISVECTOR '(' expression ')' {
  if (text_flag) {
    char *temp = new char[9 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "is_vector(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    $$ = new value (0);
    if ($3->type == DENSE) {
      if ($3->dmat->get_nrow () == 1 || $3->dmat->get_ncol () == 1)
	$$ = new value (1);
    }				// end if
    if ($3->type == SPARSE) {
      if ($3->smat->get_nrow () == 1 || $3->smat->get_ncol () == 1)
	$$ = new value (1);
    }				// end if
  }				// end else
}

| WHAT_IS '(' expression ')' {
  if (text_flag) {
    char *temp = new char[7 + 1 + strlen ($3->str) + 1];
    strcpy (temp, "what_is(");
    strcat (temp, $3->str);
    strcat (temp, ")");
    $$ = new value (temp);
    $3->free ();
    delete $3;
  }				// end if
  else {
    if ($3->type == NUM) {
      //      cout << "scalar: " << $3->num << "\n";
      sp_out("scalar: "); sp_out($3->num); sp_out("\n");
      $$ = new value ();
    }
    else {
      if ($3->type == STRING) {
	//	cout << "string: " << $3->str << "\n";
	sp_out("string: "); sp_out($3->str); sp_out("\n");
	$$ = new value ();
      }
      else {
	if ($3->type == SPARSE) {
	  int nrow = $3->smat->get_nrow();
	  int ncol = $3->smat->get_ncol();
	  //	  cout << "sparse matrix of size: " << nrow << " x " 
	  //	       << ncol << "\n";
	  sp_out("sparse matrix of size: "); sp_out(nrow);
	  sp_out(" x "); sp_out(ncol); sp_out("\n");
						    
	  $$ = new value ();	 
	}
	else {
	  if ($3->type == DENSE) {
	    int ncol = $3->dmat->get_ncol();
	    int nrow = $3->dmat->get_nrow();
	    if ($3->dmat->get_nrow () == 1) {
	      //	      cout << "vector of size: " << ncol << "\n";
	      sp_out("vector of size: "); sp_out(ncol);
	      sp_out("\n");
	    }
	    else {
	      if ($3->dmat->get_ncol () == 1) {
		//		cout << "vector of size: " << nrow << "\n";
		sp_out("vector of size: "); sp_out(nrow);
		sp_out("\n");
	      }
	      else {
		//		cout << "dense matrix of size: " << nrow << " x " 
		//		     << ncol << "\n";
		sp_out("dense matrix of size: "); sp_out(nrow);
		sp_out(" x "); sp_out(ncol); sp_out("\n");
	      }
	    }
	    $$ = new value ();
	  }
	  else {
	    //	    cout << "unknown\n";
	    sp_out("unknown\n");
	    $$ = new value ();
	  }
	}
      }
    }
  }				// end else
}

| TYPE '(' expression ')' {
  if(text_flag) {
    char *temp = new char[4 + 1 + strlen($3->str) + 1];
    sprintf(temp, "type(%s)", $3->str);
    $$ = new value(temp);
    $3->free();
    delete $3;
  }
  else {
    $$ = spit_file($3->str);
  }
}

| SPY '(' expression ')' {
  
#ifndef NO_TK_COMPILE

  if(text_flag) {
    char *temp = new char[3 + 1 + strlen($3->str) + 1];
    sprintf(temp, "spy(%s)", $3->str);
    $$ = new value(temp); 
    $3->free();
    delete $3;
  }
  else {
    if(no_tk) {
      sp_error("view command not implemented in non-tk mode...\n"
	       "please restart spmath without the -notk option to use view");
      YYERROR;
    }
    if(cur_spyview > MAX_VIEWS) {
      sp_error("too many views open at once");
      YYERROR;
    }
    if($3->type == SPARSE) {
      if($3->smat->get_nrow() != $3->smat->get_ncol()) {
	yyerror("Error, matrix parameter to view must be square");
	YYERROR;
      }
      spyviews[cur_spyview] = new view(*$3->smat);
    }
    else if($3->type == DENSE) {
      if($3->dmat->get_nrow() != $3->dmat->get_ncol()) {
	yyerror("Error, matrix parameter to view must be square");
	YYERROR;
      }
      spyviews[cur_spyview] = new view(*$3->dmat);
    }
    else {
      sp_error("parameter to view(...) must be a matrix");
      YYERROR;
    }
    spyviews[cur_spyview]->create();
    cur_spyview++;
    $$ = new value();
    $3->free();
    delete $3;
  }

#else

  sp_error("view command not implemented in non-tk version...\n");
  
  $$ = new value();
  $3->free();
  delete $3;
  
#endif

}

;



dense: '[' dense_row_list ']' {
  if (text_flag) {
    $2->init_curr_prev ();
    int len = strlen ($2->get_current ().get_value ().str);
    while (!$2->end ()) {
      $2->get_next ();
      len += strlen ($2->get_current ().get_value ().str);
    }				// end while
    char *temp = new char[len + 2];
    $2->init_curr_prev ();
    strcpy (temp, "[");
    strcat (temp, $2->get_current ().get_value ().str);
    while (!$2->end ()) {
      $2->get_next ();
      strcat (temp, $2->get_current ().get_value ().str);
    }				// end while
    strcat (temp, "]");
    $2->remove_all ();
    delete $2;
    $$ = new value (temp);
  }				// end if
  else {
    err = 0;
    $$ = process_dense_row_list ($2, err);
    delete $2;
    if (err)
      YYERROR;
  }				// end else
}

| vector {
  $$ = $1;
}

;


dense_row_list: dense_row dense_row_list {
  $2->push (entry (strdup (""), *$1));
  $$ = $2;
  delete $1;
}

| dense_row_end {
  $$ = new list;
  $$->push (entry (strdup (""), *$1));
  delete $1;
}

;

dense_row: comma_list SEMI {
  if (text_flag) {
    $1->init_curr_prev ();
    int len = strlen ($1->get_current ().get_value ().str) + 1;
    while (!$1->end ()) {
      $1->get_next ();
      len += strlen ($1->get_current ().get_value ().str) + 1;
    }				// end while
    char *temp = new char[len + 1];
    $1->init_curr_prev ();
    strcpy (temp, $1->get_current ().get_value ().str);
    while (!$1->end ()) {
      $1->get_next ();
      strcat (temp, ",");
      strcat (temp, $1->get_current ().get_value ().str);
    }				// end while
    $1->remove_all ();
    delete $1;
    strcat (temp, ";");
    $$ = new value (temp);
  }				// end if
  else {
    err = 0;
    $$ = process_comma_list ($1, err);
    delete $1;
    if (err)
      YYERROR;
  }				// end else
}

;

dense_row_end: comma_list {
  if (text_flag) {
    $1->init_curr_prev ();
    int len = strlen ($1->get_current ().get_value ().str) + 1;
    while (!$1->end ()) {
      $1->get_next ();
      len += strlen ($1->get_current ().get_value ().str) + 1;
    }				// end while
    char *temp = new char[len];
    $1->init_curr_prev ();
    strcpy (temp, $1->get_current ().get_value ().str);
    while (!$1->end ()) {
      $1->get_next ();
      strcat (temp, ",");
      strcat (temp, $1->get_current ().get_value ().str);
    }				// end while
    $1->remove_all ();
    delete $1;
    $$ = new value (temp);
  }				// end if
  else {
    err = 0;
    $$ = process_comma_list ($1, err);
    delete $1;
    if (err)
      YYERROR;
  }				// end else
}

;



comma_list: expression COMMA comma_list {
  $3->push (entry (strdup (""), *$1));
  $$ = $3;
  delete $1;
}

| expression {
  $$ = new list;
  $$->push (entry (strdup (""), *$1));
  delete $1;
}

;


vector: expression COLON expression COLON expression {
  if (text_flag) {
    char *temp =
      new char[strlen ($1->str) + 1 + strlen ($3->str) + 1 +
	       strlen ($5->str)];
    strcpy (temp, $1->str);
    strcat (temp, ":");
    strcat (temp, $3->str);
    strcat (temp, ":");
    strcat (temp, $5->str);
    $$ = new value (temp);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
  }				// end if
  else {
    err = 0;
    $$ = vec ($1, $3, $5, err);
    $1->free ();
    delete $1;
    $3->free ();
    delete $3;
    $5->free ();
    delete $5;
    if (err)
      YYERROR;
  }				// end else   
}

;

/*RAJOUT DE SUBROUTINES ===================================================
=========================================================================*/

subroutin_inc:			/* empty */
{
  text_flag += 1;
}

;

subroutin_dec:			/* empty */
{
  text_flag -= 1;
}

;


/*FIN DE RAJOUT DE SUBROUTINES ============================================
=========================================================================*/



%%

//
// sp_init()
//  
//  Prepares the spmath environment: inserts useful
// constants into the global sym table and prints out
// a welcome message.
//
void sp_init ()
{
  char buf[256];

  // Insert some useful constants into the symbol table.

  syms->insert (entry (strdup ("true"), value (1)));
  syms->insert (entry (strdup ("false"), value (0)));
  syms->insert (entry (strdup ("pi"), value (3.1415926535)));
  syms->insert (entry (strdup ("e"), value (2.718281728459)));

  strcpy(buf, spmathpath);
  strcat(buf, "header");
  spit_file(buf);

  int err = 0;

  supress_output = 1;
  
  strcpy(buf, spmathpath);
  strcat(buf, "init.sp");

  load(new value(buf), 0, err);
  
  supress_output = 0;

  if(err) {
    sp_error("Could not load solver defaults from file \"init.sp\"\n\n");
    sp_error("using internal defaults...\n\n");

    supress_output = 1;

    sp_parse_string("slvr_solver_name = \"gmres\"");
    sp_parse_string("slvr_prec_name = \"ilut\"");
    sp_parse_string("slvr_prec_args = [3,0.0001]");
    sp_parse_string("slvr_prec_type = \"right\"");
    sp_parse_string("slvr_krylov = 10");
    sp_parse_string("slvr_maxmult = 100");
    sp_parse_string("slvr_tols = [0.000001, 0.0]");

    supress_output = 0;
  }
}

//
// sp_parse (const char *input)
//
//  Begins parsing.  If spmath is in non-windowed mode,
// then control is given over to the parser fot the rest
// of the session.  In windowed mode, this function
// simply processes one command (char *input) using the
// parser and then outputs the result, returning control
// to the calling program (in this case, the "Enter"
// key handler in spmath.tcl)
//
int sp_parse (const char *input)
{
  // Buffer to be lexed and parsed
  static char buff[BUFF_SIZE];

  // Dummy variable for lexer
  YYSTYPE dummy;

  // Main loop for command-line mode
#ifndef NO_TK_COMPILE
  if(no_tk) {
#endif
    
    // If we're in command-line mode, then parse the command
    // line until the user quits.
    
    while (1) {
      // Output the prompt
      sp_prompt();
      // Get the command
      cin.getline (buff + strlen (buff), BUFF_SIZE - strlen (buff), '\n');
      *(buff + strlen (buff) + 1) = '\0';
      *(buff + strlen (buff)) = '\n';
      str_flag = 1;
      str = buff;
      nest_count = 0;

      int l;
      // Adjust the nest count using lex
      do {
	l = yylex(&dummy);
      } while(l != '\n' && l != ERROR);
      // do nothing
      
      // Catch errors and try again
      if(l == ERROR) { 

	string t(buff);
	hist_list.put_item (t);
	hist_num ++;
	hist_cur = hist_num;
	buff[0] = '\0';
	continue;
      }

      *(buff + strlen(buff) -1) = ' ';
      
      // As long as we are in a nested construct (like a for
      // loop or a function definition) then continue to read
      // into the buffer.
      while (nest_count > 0) {
	sp_prompt();
	cin.getline (buff + strlen (buff), BUFF_SIZE - strlen (buff), '\n');
	*(buff + strlen (buff) + 1) = '\0';
	*(buff + strlen (buff)) = '\n';
	str_flag = 1;
	str = buff;
	nest_count = 0;

	// Adjust the nest count using lex
	do {
	  l = yylex(&dummy);
	} while(l != '\n' && l != ERROR);
	// do nothing
	
	// Catch errors and try again
	if(l == ERROR) { 

	  string t(buff);
	  hist_list.put_item (t);
	  hist_num ++;
	  hist_cur = hist_num;
	  buff[0] = '\0';
	  continue;
	}

	*(buff + strlen (buff) - 1) = ' ';
      }				// end while

      string t (buff);
      if (strlen (t) > 1) {
	
	// Make the last character in the buffer a flag
	// (for the parser)
	*(buff + strlen (buff) - 1) = '$';
	str_flag = 1;
	str = buff;
	
	// Reset the parser and parse all the stuff we
	// sent to the lexer
	yyrestart (yyin);
	yyparse ();
	
	// Add command to history
	hist_list.put_item (t);

	// Increment the number of items in history
	hist_num ++;
	hist_cur = hist_num;
      }				// end if
    bail:

      nest_count = 0;

      // Erase the buffer
      buff[0] = '\0';
    }				// end while

#ifndef NO_TK_COMPILE
  }

  // We're in tk mode, so we're just parsing a string
  else {

    // Stick in input string into the buffer and
    // format it for lex

    strcpy (buff + strlen (buff), input);
    *(buff + strlen (buff) + 1) = '\0';
    *(buff + strlen (buff)) = '\n';

    str_flag = 1;
    str = buff;
    nest_count = 0;
    
    int l;
    // Adjust the nest count using lex
    do {
      l = yylex(&dummy);
    } while(l != '\n' && l != ERROR);
    // do nothing
    
    // Catch errors and try again
    if(l == ERROR) { 
      
      string t(buff);
      hist_list.put_item (t);
      hist_num ++;
      hist_cur = hist_num;
      sp_out("\n");
      sp_prompt();

      // Erase the buffer
      buff[0] = '\0';
      return 0;
    }

    // Eliminate the newline character
    *(buff + strlen (buff) - 1) = ' ';
    
    if (nest_count > 0) {
      // Respond that we are waiting for additional input
      sp_prompt();
      return 1;		
    }
    
    string t (buff);
    if (strlen (t) > 1) {
      
      // Add command to history
      hist_list.put_item (t);
      
      // Make the last character in the buffer a flag
      // (for the parser)
      *(buff + strlen (buff) - 1) = '$';
      str_flag = 1;
      str = buff;
      
      // Reset the parser and parse from the string
      yyrestart (yyin);
      yyparse ();

      // Increment the number of items in history
      hist_num += 1;
      hist_cur = hist_num;
    }

    sp_out("\n");
    sp_prompt();

    // Erase the buffer
    buff[0] = '\0';
  }
#endif

  // Only tk mode will reach the end
  return 0;
}

//
// sp_parse_string(char * input)
//
//  This function is intended to be a behind-the-scenes
// helper function for quickly executing spmath commands.
// Simply pass this function the command you would like
// to execute, and it will be parsed as usual without
// outputting anything to the command line.  This fucntion
// will not handle multi-line statements with nested
// constructs.
//
// ex: sp_parse_string("pi=3.14");
//     sp_parse_string("solver_name=\"gmres\"");
//     etc...
//
int sp_parse_string(const char * input) {

  char buf[256];

  // Format string for parser
  strcpy(buf, input);
  strcat(buf, "$");

  // Set the string flag and let the string be the
  // input.
  str_flag = 1;
  str = buf;
  nest_count = 0;

  // Restart the parser and run it on the string.
  yyrestart (yyin);
  yyparse ();

  // Return success
  return 1;
}

//
// yyerror(char *err)
//
//  Overrides the default yyerror and re-routes the
// output to our more versitile function sp_error.
// It is overloaded to accept const char *'s so that
// picky compilers don't give warning messages about
// string literals being passed as char *'s.
//
int yyerror (char *err) {

  sp_error (err);
  sp_error ("\n");
  return -1;
}

int yyerror (const char *err) {

  sp_error (err);
  sp_error ("\n");
  return -1;
}

value *eval_funct (char *name, list * args, int &err) {

  // Evaluate a user defined function with the name mame and an
  // argument list args.

  syms = new symtab (syms);

  // Lookup the function and get its object
  funct_def f (*root.lookup (name).get_value ().funct);

  // Make sure the arglist has the right number of arguments
  int count = 0;
  args->init_curr_prev ();
  if (!args->empty ()) {
    count += 1;
    while (!args->end ()) {
      args->get_next ();
      count += 1;
    }				// end while
  }				// end if

  if (count != f.get_argc ()) {
    err = 1;
    sp_error("Error, function \"");
    sp_error(name);
    sp_error("\" takes ");
    sp_error(f.get_argc());
    sp_error(" argument");
    if(f.get_argc() != 1) {
      sp_error("s");
    }
    sp_error("\n");
    return new value ();
  }				// end if

  int i;
  // Add all arguments to the local symbol table
  for (i = 1; i <= f.get_argc (); i += 1) {
    syms->insert (entry (f.get_arg (i), args->pop ().get_value ()));
  }				// end for

  str_flag = 1;
  str = new char[strlen (f.get_body ()) + 1];
  strcpy (str, f.get_body ());
  strcat (str, "$");

  //  cout << "str is " << str << endl;

  // Restart the parser using the function text as the input
  yyrestart (yyin);
  yyparse ();

  value *ret;
  // Lookup the return value in the symbol table
  if (syms->contains (f.get_ret ())) {
    ret = new value (syms->lookup (f.get_ret ()).get_value ());
    // Determine the type and return the appropriate value
    if (ret->type == NUM)
      ret->num = (ret->num);
    else if (ret->type == DENSE)
      ret->dmat = new dense (*ret->dmat);
    else if (ret->type == SPARSE)
      ret->smat = new sparse (*ret->smat);
    else if (ret->type == STRING)
      ret->str = strdup (ret->str);
    else
      assert (0);		// Should never happen
  }				// end if  
  else
    // Return value was not in the symbol table
    ret = new value ();

  // Asserts that this funtion's symbol table is not the global
  // symbol table
  assert (!syms->is_root ());
  syms->remove_all ();
  symtab *temp = syms;

  // Reestablish sysms as the parent's symbol table
  syms = syms->get_parent ();
  delete temp;

  // Return the function's return value
  return ret;

}				// end  eval_funct

//
// value *load (value * file, int echo, int &err)
//
//  This function will load a file containing spmath
// commands from a file whose filename is the string
// wrapped by value * file.  It then executes these
// commands one by one and, if echo == 1, also echos
// the commands onto the command line.  It is assumed
// that each command appears on a separate line
// (with the exception of nested constructs, which
// may span several lines).
//

value *load (value * file, int echo, int &err)
{
  char buff[BUFF_SIZE];		// input buffer

  buff[0] = '\0';

  YYSTYPE dummy;
  struct stat statbuff;		// buffer to hold the status of the file

  char * filename;

  if (file->type != STRING) {
    yyerror ("Error, 1st argument of load must be a string.");
    err = 1;
    return file;
  }				// end if

  if (stat (file->str, &statbuff) != 0) {
    char second_try[128];
    sprintf(second_try, "%s/%s/%s", spmathpath, SCRIPT_DIR_NAME, file->str);

    if (stat (second_try, &statbuff) != 0) {

      yyerror ("Error, file does not exist.");
      err = 1;
      return file;
    }
    else {
      filename = second_try;
    }
  }
  else {
    filename = file->str;
  }

  ifstream iFile (filename, ios::in);

  if(echo) {
    sp_out("Executing \"");
    sp_out(filename);
    sp_out("\"...\n\n");
    sp_prompt();
  }

  while (!iFile.eof ()) {

    // Get the next line from the file and then format
    // it for the lexer

    iFile.getline (buff + strlen (buff), BUFF_SIZE - strlen (buff), '\n');
    *(buff + strlen (buff) + 1) = '\0';
    *(buff + strlen (buff)) = '\n';
    str_flag = 1;
    str = buff;
    nest_count = 0;

    // Skip blank lines and commented lines

    if(str[0] == '\n' || str[0] == '#') {

      buff[0] = '\0';
      continue;
    }

    // Lex this part (without then passing control to
    // the parser)

    int l;
    // Adjust the nest count using lex
    do {
      l = yylex(&dummy);
    } while(l != '\n' && l != ERROR);
    // do nothing

    // Catch errors and try again
    if(l == ERROR) { 
      
      buff[0] = '\0';
      continue;
    }

    *(buff + strlen (buff) - 1) = ' ';

    // If we're still in the middle of a nested construct
    // then keep lexing it until it ends

    while (nest_count > 0) {

      // Get the next line of this nested construct

      iFile.getline (buff + strlen (buff), BUFF_SIZE - strlen (buff), '\n');
      *(buff + strlen (buff) + 1) = '\0';
      *(buff + strlen (buff)) = '\n';
      str_flag = 1;
      str = buff;
      nest_count = 0;

      // Lex this part (without then passing control to
      // the parser)

      do {
	l = yylex(&dummy);
      } while(l != '\n' && l != ERROR);
      // do nothing
      
      // Catch errors and try again
      if(l == ERROR) { 
	
	buff[0] = '\0';
	continue;
      }

      *(buff + strlen (buff) - 1) = ' ';
    }				// end while

    // If echo mode is on, output the command so that
    // the user can see it (but don't add it to the
    // history)
    if(echo) {
      sp_prompt();
      sp_command(str);
      sp_out("\n");
    }

    // Now format the string for the yacc parser

    *(buff + strlen (buff) - 1) = '$';
    str_flag = 1;
    str = buff;

    // Run the yacc parser on the buffer string

    yyrestart(yyin);
    yyparse();
    buff[0] = '\0';

    // Put whitespace between outputs from the spmath
    // statements

    sp_out("\n");
  }				// end while

 bail:

  return new value ();
}				// end load

value *save (value * texte, value * filename, int &err)
{
  struct stat statbuff;		// buffer to hold the status of the file

  if (filename->type != STRING) {
    yyerror ("Error, 2nd argument of save must be a string.");
    err = 1;
    return filename;
  }				// end if

  if (stat (filename->str, &statbuff) == 0) {
    yyerror ("Error, file already exists.");
    err = 1;
    return filename;
  }				// end if

  if (texte->type != STRING) {
    yyerror ("Error, 1st argument of save must be a string.");
    err = 1;
    return texte;
  }				// end if

  char *txt = texte->str;
  //  cout << "txt: " << txt[0] << txt[1] << txt[2];
  ofstream destination (filename->str);
  if (!destination) {
    yyerror ("Error opening destination file");
    err = 1;
    return (filename);
  }
  int i = 0;
  while (txt[i] != '$') {
    destination << txt[i];
    i++;
  }


/* Why is this err? */
  return new value (err);

}				// end save

value *process_comma_list (list * set, int &err)
{
  value *val;
  int ncol = 0;
  int nrow = 0;
  int i, j;

  if (set->top ().get_value ().type == NUM) {
    ncol = 1;
    nrow = 1;
  }				// end if
  else {
    if (set->top ().get_value ().type == DENSE) {
      ncol = set->top ().get_value ().dmat->get_ncol ();
      nrow = set->top ().get_value ().dmat->get_nrow ();
    }				// end if
    else {
      if (set->top ().get_value ().type == SPARSE) {
	ncol = set->top ().get_value ().smat->get_ncol ();
	nrow = set->top ().get_value ().smat->get_nrow ();
      }				// end if
    }				// end else
  }				// end else

  set->init_curr_prev ();
  while (!set->end ()) {
    set->get_next ();
    if (set->get_current ().get_value ().type == NUM) {
      ncol += 1;
      // A number is considered a matrix of dimensions 1 x 1
      if (nrow != 1) {
	yyerror ("Error, Ilegal dimensions for matrix.");
	set->remove_all ();
	err = 1;
	return new value ((0));
      }				// end if
    }				// end if
    else {
      if (set->get_current ().get_value ().type == DENSE) {
	ncol += set->get_current ().get_value ().dmat->get_ncol ();
	if (nrow != set->get_current ().get_value ().dmat->get_nrow ()) {
	  yyerror ("Error, Ilegal dimensions for matrix.");
	  set->remove_all ();
	  err = 1;
	  return new value ((0));
	}			// end if
      }				// end if
      else {
	if (set->get_current ().get_value ().type == SPARSE) {
	  ncol += set->get_current ().get_value ().smat->get_ncol ();
	  if (nrow != set->get_current ().get_value ().smat->get_nrow ()) {
	    yyerror ("Error, Ilegal dimensions for matrix.");
	    set->remove_all ();
	    err = 1;
	    return new value ((0));
	  }			// end if
	}			// end if
      }				// end else
    }				// end else
  }				// end while


  dense *d;
  d = new dense (nrow, ncol);
  ncol = 0;
  while (!set->empty ()) {
    if (set->top ().get_value ().type == NUM) {
      ncol += 1;
      d->set_element (1, ncol, set->top ().get_value ().num);
    }				// end if
    else {
      if (set->top ().get_value ().type == DENSE) {
	for (i = 1; i <= nrow; i += 1)
	  for (j = ncol + 1;
	       j <= ncol + set->top ().get_value ().dmat->get_ncol (); j += 1)
	    d->set_element (i, j,
			    set->top ().get_value ().dmat->get_element (i,
									j -
									ncol));
	ncol += set->top ().get_value ().dmat->get_ncol ();
      }				// end if
      else {
	if (set->top ().get_value ().type == SPARSE) {
	  for (i = 1; i <= nrow; i += 1)
	    for (j = ncol + 1;
		 j <= ncol + set->top ().get_value ().smat->get_ncol ();
		 j +=
		 1) d->set_element (i, j,
				    set->top ().get_value ().smat->
				    get_element (i, j - ncol));
	  ncol += set->top ().get_value ().smat->get_ncol ();
	}			// end if
      }				// end else
    }				// end else
    set->remove_top ();
  }				// end while

  val = new value (d);
  return val;

}				// end process_comma_list



value *process_dense_row_list (list * set, int &err)
{
  value *val;
  int ncol = 0;
  int nrow = 0;
  int i, j;

  if (set->top ().get_value ().type == NUM) {
    ncol = 1;
    nrow = 1;
  }				// end if
  else {
    if (set->top ().get_value ().type == DENSE) {
      ncol = set->top ().get_value ().dmat->get_ncol ();
      nrow = set->top ().get_value ().dmat->get_nrow ();
    }				// end if
    else {
      if (set->top ().get_value ().type == SPARSE) {
	ncol = set->top ().get_value ().smat->get_ncol ();
	nrow = set->top ().get_value ().smat->get_nrow ();
      }				// end if
    }				// end else
  }				// end else

  set->init_curr_prev ();
  while (!set->end ()) {
    set->get_next ();
    if (set->get_current ().get_value ().type == NUM) {
      nrow += 1;
      if (ncol != 1) {
	yyerror ("Error, Ilegal dimensions for matrix.");
	set->remove_all ();
	err = 1;
	return new value ((0));
      }				// end if
    }				// end if
    else {
      if (set->get_current ().get_value ().type == DENSE) {
	nrow += set->get_current ().get_value ().dmat->get_nrow ();
	if (ncol != set->get_current ().get_value ().dmat->get_ncol ()) {
	  yyerror ("Error, Ilegal dimensions for matrix.");
	  set->remove_all ();
	  err = 1;
	  return new value ((0));
	}			// end if
      }				// end if
      else {
	if (set->get_current ().get_value ().type == SPARSE) {
	  nrow += set->get_current ().get_value ().smat->get_nrow ();
	  if (ncol != set->get_current ().get_value ().smat->get_ncol ()) {
	    yyerror ("Error, Ilegal dimensions for matrix.");
	    set->remove_all ();
	    err = 1;
	    return new value ((0));
	  }			// end if
	}			// end if
      }				// end else
    }				// end else
  }				// end while


  dense *d;
  d = new dense (nrow, ncol);
  nrow = 0;
  while (!set->empty ()) {
    if (set->top ().get_value ().type == NUM) {
      nrow += 1;
      d->set_element (nrow, 1, set->top ().get_value ().num);
    }				// end if
    else {
      if (set->top ().get_value ().type == DENSE) {
	for (i = 1; i <= ncol; i += 1)
	  for (j = nrow + 1;
	       j <= nrow + set->top ().get_value ().dmat->get_nrow (); j += 1)
	    d->set_element (j, i,
			    set->top ().get_value ().dmat->get_element (j -
									nrow,
									i));
	nrow += set->top ().get_value ().dmat->get_nrow ();
      }				// end if
      else {
	if (set->top ().get_value ().type == SPARSE) {
	  for (i = 1; i <= ncol; i += 1)
	    for (j = nrow + 1;
		 j <= nrow + set->top ().get_value ().smat->get_nrow ();
		 j +=
		 1) d->set_element (j, i,
				    set->top ().get_value ().smat->
				    get_element (j - nrow, i));
	  nrow += set->top ().get_value ().smat->get_nrow ();
	}			// end if
      }				// end else
    }				// end else
    set->remove_top ();
  }				// end while

  val = new value (d);
  return val;

}				// end process_dense_row_list

//
// These functions are defined to be extern "C" so that they
// can be passed to the "function" function in functions.C
// along with the regular C library trig functions (cos, sin,
// etc...)
//

extern "C" double cot (double x)
{
  return 1 / tan (x);
}

extern "C" double csc (double x)
{
  return 1 / sin (x);
}

extern "C" double sec (double x)
{
  return 1 / cos (x);
}

extern "C" double acot (double x)
{
  return atan (1 / x);
}

extern "C" double acsc (double x)
{
  return asin (1 / x);
}

extern "C" double asec (double x)
{
  return acos (1 / x);
}

extern "C" double round (double x)
{
  return double (int (x + .5));
}

extern "C" double trunc (double x)
{
  return double (int (x));
}

extern "C" double rnd (double x)
{
  // Return a random number (double) between 0 and 1
  // The variable x is just a dummy variable, so that this
  // function can be used to be applied to a dense matrix.

  static int rnd_flag = 0;
  time_t *tloc = NULL;

  // So that compilers won't complain...
  x = 0;

  // The first time this routine is executed, set the seed
  // the the current time.  This prevents the same sequence from
  // being used twice.
  if (rnd_flag == 0) {
    srand (int (time (tloc)));
    rnd_flag = 1;
  }
  return (double (rand ()) / 32767.0);

}				// end rnd


